package inria.paperoles.gestures;

import inria.paperoles.gestures.SingleInkStroke.Peek;
import inria.paperoles.grammar.Token;
import inria.paperoles.scoremodel.ScorePart;
import inria.paperoles.scoremodel.VoiceLine;

import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Area;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Vector;

import edu.umd.cs.piccolo.util.PBounds;

import papertoolkit.pen.PenSample;
import papertoolkit.pen.ink.InkStroke;

public class ComplexInkStroke extends GeneralInkStroke {
	
	//if arrow, the head is the first stroke in the vector
	Vector<SingleInkStroke> strokes;
		
	public ComplexInkStroke(){
		super();
		
		strokes = new Vector<SingleInkStroke>();
	}	
	
	public ComplexInkStroke(SingleInkStroke stroke){
		super(stroke.getSamples());
		
		strokes = new Vector<SingleInkStroke>();
		strokes.add(stroke);
	}	
	
	public void addStroke(SingleInkStroke stroke){
		strokes.add(stroke);
		
		for(PenSample sample:stroke.getSamples()){
			addSample(sample);
		}
	}
	
	public void setToken(Token token){
		associatedToken = token;
		
		for(SingleInkStroke stroke:strokes)
			stroke.setToken(token);
	}
	
	
	public void addStroke(GeneralInkStroke stroke){
		if(stroke instanceof SingleInkStroke)
			addStroke((SingleInkStroke)stroke);
		
		else 
			for(SingleInkStroke singleStroke:((ComplexInkStroke)stroke).strokes){
				strokes.add(singleStroke);
			
				for(PenSample sample:singleStroke.getSamples()) addSample(sample);
			}
	}
	
	public Vector<SingleInkStroke> getStrokes(){
		return strokes;
	}
	
	public Vector<Shape> getShapes(){
		Vector<Shape> shapes = new Vector<Shape>();
		
		for(SingleInkStroke stroke: strokes)
			shapes.add(stroke.getShape());
		
		return shapes;
	}
	
	public Vector<Shape> getSmoothShapes(){
		Vector<Shape> shapes = new Vector<Shape>();
		
		for(SingleInkStroke stroke: strokes)
			shapes.add(stroke.getSmoothShape());
		
		return shapes;
	}
	
	public Rectangle2D getBounds(){
		Rectangle2D bounds = null;
		
		for(SingleInkStroke stroke: strokes){
			if(bounds == null){
				bounds = new Rectangle2D.Double();
				bounds.setRect(stroke.getBounds());
			}
			else bounds.add(stroke.getBounds());
		}
		
		return bounds;
	}

	@Override
	public Shape getShape() {
		Area shape = new Area();
		for(SingleInkStroke stroke: strokes)
			shape.add(new Area(stroke.getShape()));
		
		return shape;
	}

	@Override
	public Shape getSmoothShape() {
		Area shape = new Area();
		for(SingleInkStroke stroke: strokes)
			shape.add(new Area(stroke.getSmoothShape()));
		
		return shape;
	}
	
	@Override
	public boolean intersects(PBounds bounds) {
		for(SingleInkStroke stroke: strokes)
			if(stroke.getShape().intersects(bounds)) return true;
		
		return false;
	}

	@Override
	public SingleStrokeAnalysisResult getAnalysisResult() {
		// TODO Auto-generated method stub
		return null;
	}

	
	// I need to rewrite these functions... they are not ready... what about the samples!!!!! althought  they will not be used afterallll :(
	@Override
	public GeneralInkStroke getMirroredStroke(int axis) {
		ComplexInkStroke inkStroke = new ComplexInkStroke();
		
		for(SingleInkStroke stroke: strokes)
			inkStroke.addStroke((SingleInkStroke)stroke.getMirroredStroke(axis));
		
		return inkStroke;
	}

	@Override
	public GeneralInkStroke getScaledStroke(double scalex, double scaley) {
		ComplexInkStroke inkStroke = new ComplexInkStroke();
		
		for(SingleInkStroke stroke: strokes)
			inkStroke.addStroke((SingleInkStroke)stroke.getScaledStroke(scalex, scaley));
		
		return inkStroke;
	}

	@Override
	public GeneralInkStroke getRotatedStroke(double radians) {
		ComplexInkStroke inkStroke = new ComplexInkStroke();
		
		for(SingleInkStroke stroke: strokes)
			inkStroke.addStroke((SingleInkStroke)stroke.getRotatedStroke(radians));
		
		return inkStroke;
	}
	
	@Override
	public GeneralInkStroke getTimeInversedStroke() {
		// I just inverse the samples in every stroke separetely
		// i do not change the real order of the strokes
		ComplexInkStroke inkStroke = new ComplexInkStroke();
		
		for(SingleInkStroke stroke: strokes)
			inkStroke.addStroke((SingleInkStroke)stroke.getTimeInversedStroke());
		
		return inkStroke;
	}

	@Override
	public GeneralInkStroke getWithCutEnds(int num1, int num2) {
		ComplexInkStroke inkStroke = new ComplexInkStroke();
		
		for(SingleInkStroke stroke: strokes)
			inkStroke.addStroke((SingleInkStroke)stroke.getWithCutEnds(num1, num2));
		
		return inkStroke;
	}

	@Override
	public GeneralInkStroke getRandomStroke() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean intersects(Line2D line) {
		for(SingleInkStroke stroke: strokes)
			if(stroke.intersects(line)) return true;
		
		return false;
	}

	@Override
	public
	boolean endsTo(GeneralInkStroke endstroke) {
		for(SingleInkStroke stroke: strokes)
			if(stroke.endsTo(endstroke)) return true;
		
		return false;
	}

	@Override
	public boolean startsFrom(GeneralInkStroke startstroke) {
		for(SingleInkStroke stroke: strokes)
			if(stroke.startsFrom(startstroke)) return true;
		
		return false;
	}

	@Override
	public boolean inShape(Shape shape) {
		for(SingleInkStroke stroke: strokes)
			if(stroke.inShape(shape)) return true;
		
		return false;
	}

	@Override
	public int getPigTailIndex() {
		return strokes.lastElement().getPigTailIndex();
	}

	@Override
	public boolean endsAbout(GeneralInkStroke endstroke) {
		for(SingleInkStroke stroke: strokes)
			if(stroke.endsAbout(endstroke)) return true;
		
		return false;
	}

	@Override
	public boolean startsAbout(GeneralInkStroke startstroke) {
		for(SingleInkStroke stroke: strokes)
			if(stroke.startsAbout(startstroke)) return true;
		
		return false;
	}

	@Override
	public ScorePart getNote(VoiceLine vline) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public ScorePart[] getNotes(VoiceLine vline) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public double distanceFrom(GeneralInkStroke generalInkStroke) {
		double minDist = 5000;
		
		//better consider SQR distances
		for(SingleInkStroke stroke:strokes){
			double dist = stroke.distanceFrom(generalInkStroke);
			
			if(dist<minDist) minDist = dist;
		}	
		
		return minDist;

	}
	
}
