package inria.paperoles.gestures;

import inria.paperoles.grammar.Token;
import inria.paperoles.scoremodel.ScorePart;
import inria.paperoles.scoremodel.VoiceLine;

import java.awt.Color;
import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import edu.umd.cs.piccolo.util.PBounds;

import papertoolkit.pen.PenSample;
import papertoolkit.pen.ink.InkStroke;
import papertoolkit.units.Units;

public abstract class GeneralInkStroke extends InkStroke{

	final static int ENDS_SIZE = 5;
	
	public final static int X_AXIS = 1;
	public final static int Y_AXIS = 2;
	
	public final static Color STROKE_COLOR = new Color(20, 20, 250);

	public static Color currentColor = STROKE_COLOR;
	Color stroke_color;
	
	public GeneralInkStroke(List<PenSample> samples) { 
		super(samples);
		
		stroke_color = currentColor;

	}
	
	public GeneralInkStroke() { 
		super();
		
		stroke_color = currentColor;
	}
	
	Token associatedToken;
	Shape smoothShape;
	
	public GeneralInkStroke(Collection<PenSample> samples, Units referenceUnit) {
		super(samples, referenceUnit);

	}

	public abstract Collection<SingleInkStroke> getStrokes();

	
	public GeneralInkStroke(InkStroke stroke) {
		this(stroke.getSamples());
	}
	
	final static int SAMPLE_X = 1;
	
	abstract public Shape getShape();	
	abstract public Shape getSmoothShape();
	abstract public Collection<Shape> getShapes();
	
	public Color getColor(){
		return STROKE_COLOR;
	}
	
	public void setColor(Color color){
		stroke_color = color;
	}
	
	public Point2D getFirstPoint(){
		return new Point2D.Double(getStart().getX(), getStart().getY());
	}
	
	public Point2D getLastPoint(){
		return new Point2D.Double(getEnd().getX(), getEnd().getY());
	}
	

	public void setToken(Token token){
		associatedToken = token;
	}
	
	public Token getToken(){
		return associatedToken;
	}
	
	public double getSurface(){
		List<PenSample>  samples = getSamples();
		
		PenSample p1 =  samples.get(0);
		PenSample p2 = samples.get(samples.size() - 1);
		
		double x_sqr = Math.pow(p1.x - p2.x, 2);
		double y_sqr = Math.pow(p1.y - p2.y, 2);
				
		// angle (-p/2...p/2) of the line connecting the end points of the curve
	
		
		double surface = 0; 

		for(int i=0; i<samples.size(); ++i){
			PenSample sample = samples.get(i);
			
			double y = ((p2.x - sample.x)*(p2.y - p1.y)*(p1.x - p2.x) 
					+ p2.y*x_sqr + sample.y*y_sqr)/(x_sqr + y_sqr);
			
			double x = sample.x + (y - sample.y)*(p2.y - p1.y)/(p1.x - p2.x);
				
			////now compute the surface between the stroke and the connecting line
			double x1,x2,y1,y2;
			if(i==0){
				x1 = x;
				y1 = y;
			}
			else {
				x1 = x + (samples.get(i-1).x - x)/2;
				y1 = y - (y - samples.get(i-1).y)/2;
			}
			
			if(i==samples.size() - 1){
				x2 = x;
				y2 = y;
			}
			else {
				x2 = x - (x - samples.get(i+1).x)/2;
				y2 = y + (samples.get(i+1).y - y)/2;
			}
			
			/// Find the direction of 		
			
			double dsurface = Math.sqrt((Math.pow(x - sample.x, 2) + Math.pow(y - sample.y, 2))*
					(Math.pow(x1 - x2, 2) + Math.pow(y1 - y2, 2)));
			
			if(p1.y == p2.y){ // connecting line is horizontal 
				if(p1.y + (p1.y - p2.y)*(sample.x - p1.x)/(p1.x - p2.x) >= sample.y) surface -= dsurface;
				else surface += dsurface;
			}
			else { 	// otherwise
				if(p2.x + (p2.y - sample.y)*(p1.x - p2.x)/(p2.y - p1.y) >= sample.x) surface -= dsurface;
				else surface += dsurface;
			}
			
		}
		return surface;
	}
	
	// ok this will also find the direction of the angle....
	public double getDirection(){
		List<PenSample>  samples = getSamples();
		
		PenSample p1 =  samples.get(0);
		PenSample p2 = samples.get(samples.size() - 1);
		
		double fi = Math.atan((p2.y - p1.y)/(p1.x - p2.x));
		
		double surface = getSurface();
		
		// OK I may need to use the surface information later....
		/// for the moment i just return the direction...
		if(fi == 0){
			if(surface <=0) fi = -Math.PI/2;
			else fi = Math.PI/2;
		}
		else if(fi > 0){
			if(surface <= 0) fi -= Math.PI/2;
			else fi += Math.PI/2;
		} 
		else {
			if(surface<=0) fi += Math.PI/2;
			else fi -= Math.PI/2;
		}
			
		return Math.toDegrees(fi);
	}	
	
	
	public int getStartPointAngle(){ 
		List<PenSample>  samples = getSamples();
		List<PenSample> points = samples.subList(0, Math.min(ENDS_SIZE, samples.size() - 1));
		
		double fi1 = 0, fi2 = 0;
		
		PenSample point1 = points.get(0);
		
		double mina = 180;
		//make sure that I don't average discontinuous degree values
		for(int i = 1; i < points.size(); ++i){
			PenSample point2 = points.get(i);
			
			double tmpf = Math.toDegrees(Math.atan2((point2.y - point1.y), (point1.x - point2.x)));
			mina = Math.min(Math.abs(tmpf), mina);			
			fi1 += tmpf;
			
			if(tmpf < 0) tmpf+=360;
			fi2 += tmpf;
			
			point1 = point2;
		}
	
		if(mina > 45){ 
			fi2 = Math.round(fi2/(points.size() - 1));
			
			return (int)Math.round((fi2 > 180 ? (fi2 - 360) :fi2));
		}
		else return (int)Math.round(fi1/(points.size() - 1));
	}

	
	public int getEndPointAngle(){
		List<PenSample>  samples = getSamples();
		List<PenSample> points = samples.subList(Math.max(0, samples.size() - 1 - ENDS_SIZE), samples.size() - 1);

		double fi1 = 0, fi2 = 0;
		
		PenSample point1 = points.get(points.size()-1);

		double mina = 180;
		//make sure that I don't average discontinuous degree values		
		for(int i = points.size() - 2; i >=0; --i){
			PenSample point2 = points.get(i);
			
			double tmpf = Math.toDegrees(Math.atan2((point2.y - point1.y), (point1.x - point2.x)));
			mina = Math.min(Math.abs(tmpf), mina);			
			fi1 += tmpf;
			
			if(tmpf < 0) tmpf+=360;
			fi2 += tmpf;
			
			point1 = point2;
		}
			
		if(mina > 45){ 
			fi2 = Math.round(fi2/(points.size() - 1));
			
			return (int)Math.round((fi2 > 180 ? (fi2 - 360) :fi2));
		}
		else return (int)Math.round(fi1/(points.size() - 1));
	}

	abstract public boolean intersects(PBounds bounds);	
	abstract public boolean intersects(Line2D line);
	abstract public boolean startsFrom(GeneralInkStroke stroke);
	public abstract boolean endsTo(GeneralInkStroke stroke);
	
	abstract public boolean startsAbout(GeneralInkStroke stroke);
	public abstract boolean endsAbout(GeneralInkStroke stroke);
	
	abstract public Rectangle2D getBounds();

	abstract public SingleStrokeAnalysisResult getAnalysisResult();
		
	abstract public GeneralInkStroke getTimeInversedStroke();
	abstract public GeneralInkStroke getScaledStroke(double scalex, double scaley);
	abstract public GeneralInkStroke getMirroredStroke(int axis); 	
	abstract public GeneralInkStroke getWithCutEnds(int num1, int num2); 
	abstract public GeneralInkStroke getRandomStroke();
	
	abstract public boolean inShape(Shape shape);

	public abstract GeneralInkStroke getRotatedStroke(double radians);

	public abstract int getPigTailIndex();

	public abstract ScorePart[] getNotes(VoiceLine vline);
	public abstract ScorePart getNote(VoiceLine vline);

	public abstract double distanceFrom(GeneralInkStroke generalInkStroke);
		
}
