package inria.paperoles.gestures;


import inria.paperoles.rendering.StrokeRenderer;
import inria.paperoles.scoping.ScopeConstraint;

import inria.paperoles.scoremodel.Chord;
import inria.paperoles.scoremodel.GestureNote;
import inria.paperoles.scoremodel.GraphVoiceLine;
import inria.paperoles.scoremodel.Measure;
import inria.paperoles.scoremodel.MultiplePageSkeleton;
import inria.paperoles.scoremodel.Note;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.ScoreSkeleton;
import inria.paperoles.scoremodel.ScoreVerticalPointer;
import inria.paperoles.scoremodel.Segment;
import inria.paperoles.scoremodel.WaveLine;

import inria.paperoles.scoremodel.VoiceLine;
import inria.paperoles.ui.GlobalParameters;
//import inria.paperoles.ui.Scope;


import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.List;
import java.util.Vector;


public class StrokeAnalyzer{
	 
	StrokeRecognizer dollarRecognizer;
//	StrokeRenderer renderer;

	//Vector<AnalyzedStroke> strokes;
	boolean notationMode = GlobalParameters.IS_IN_NOTATION_MODE;
	
	
	public StrokeAnalyzer(){
		dollarRecognizer = new StrokeRecognizer(true);
	}

	/* not used for now*/
	public Note processNotationGesture(SingleInkStroke stroke, ScoreSkeleton skeleton){
		StrokeRenderer renderer = new StrokeRenderer(stroke);
		stroke.setShape(renderer.getShape());
	
		//////////////////////////////////////
		Rectangle2D bounds = stroke.getBounds();

		double dist;
		double mindist = 50000;	
		
		PolyStaff closeStaff = null;
		
		for(PolyStaff polyStaff:skeleton.getPolyStaffs()){
			dist = Math.abs(bounds.getCenterY() - polyStaff.getBounds().getCenterY());
			
			if(dist < mindist){
				mindist = dist;
				closeStaff = polyStaff;
			}
		}
			
		Segment segment = closeStaff.getFirstMeasure().getFirstSegment();
		Chord chord = new Chord(segment, (int)bounds.getMinX(), (int)bounds.getMinY(), (int)bounds.getWidth(), (int)bounds.getHeight());
		
		Note note;

		if(stroke.getAnalysisResult().getGraphline() ==null){
//			if(stroke.getAnalysisResult().isScoreDelimiter()) 
			note = new GestureNote(chord, stroke);
//			else note = new GraphicalMeasure(chord, stroke);
		}
		else note = new WaveLine(chord, stroke);
			
		skeleton.addSymbol(note);
		//GestureNote gnote = new GestureNote(chord, stroke);

		///////////////////////////////////
		
		return note;
	}
	
	public void performAnalysis(SingleInkStroke stroke, ScoreSkeleton skeleton){
		StrokeRenderer renderer = new StrokeRenderer(stroke);
		stroke.setShape(renderer.getShape());
	//	stroke.setSmoothShape(renderer.getSmoothShape());
	
		SingleStrokeAnalysisResult result = new SingleStrokeAnalysisResult(stroke);
	
		result.setClass(classifyGesture(stroke));
		result.setAngle(stroke.getDirection());
	
		findAssociatedStaffs(result,skeleton);
//		createConstraints(result);
	
		stroke.setAnalysisResult(result);
	}
	

	
	private short classifyGesture(GeneralInkStroke stroke){
		
		StrokeRecognitionResult result = dollarRecognizer.recognize(stroke);
//	System.err.println("... " + result.name);
		int strokeClass = result.getType();
		if(strokeClass == StrokeRecognizer.CLOSED_LINE)
			return SingleStrokeAnalysisResult.CLOSED_SELECTION;
		else return SingleStrokeAnalysisResult.GENERAL_CURVE;
	}	
		
	
	private void findAssociatedStaffs(SingleStrokeAnalysisResult result, 
			ScoreSkeleton pageSkeleton){
		
	// TODO .... include a list of all the possible combinations of useful lines.	
		Vector<VoiceLine> intersectinglines = new Vector<VoiceLine>();
		VoiceLine underline = null, overline = null, closeline = null;
		
		GeneralInkStroke stroke = result.getStroke();
		Rectangle2D bounds = stroke.getBounds();

		double dist;
		double mindist = 50000;	
		
		for(PolyStaff polyStaff:pageSkeleton.getPolyStaffs())
			for(VoiceLine voiceLine:polyStaff.getVoiceLines()){
				if(voiceLine instanceof GraphVoiceLine 
						&& voiceLine.getBounds().contains(stroke.getBounds())){
					
					result.setGraphStaff((GraphVoiceLine)voiceLine);
					return;
				}
				
				if(stroke.intersects(voiceLine.getBounds())) 
					intersectinglines.add(voiceLine);
				
				dist = Math.abs(bounds.getCenterY() - voiceLine.getBounds().getCenterY());
			
				if(dist < mindist){
					mindist = dist;
					closeline = voiceLine;
				}
			}
			
		if(stroke.getBounds().getCenterY() <= closeline.getMinY()){
			underline = closeline;
			overline = closeline.getPreviousVoiceLine();
		}
		else if(stroke.getBounds().getCenterY() >= closeline.getMaxY()){
			overline = closeline;
			underline = closeline.getNextVoiceLine();
		}
		else underline = overline = closeline;
		
		result.setIntersected(intersectinglines);
		result.setCloseLine(closeline);
		result.setOverLine(overline);
		result.setUnderLine(underline);
		
	}
	
	public boolean inNotationMode(){
		return notationMode;
	}

	public void setToNotationMode() {
		notationMode = true;
	}
	
	public void setToAnnotationMode() {
		notationMode = false;
	}
	
}
