package inria.paperoles.grammar;

import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.scoping.ScopeInstance;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.Voice;
import inria.paperoles.scoremodel.VoiceLine;

import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;

public class BlockToken extends Token{

	boolean overNotes = false;
	
	ScopeDelimiter leftDelimiter;
	ScopeDelimiter rightDelimiter;
	
	public BlockToken (ScopeDelimiter delimiter1, ScopeDelimiter delimiter2) {
		super(null, null);
		
		gcanvases.add(delimiter1.getCanvas());
		gcanvases.add(delimiter2.getCanvas());
		
		delimiter1.setParent(this);
		delimiter2.setParent(this);
		
		if(delimiter1.getStartPointer().isBefore(delimiter2.getStartPointer())){
			leftDelimiter = delimiter1;
			rightDelimiter = delimiter2;
		}
		else {
			rightDelimiter = delimiter1;
			leftDelimiter = delimiter2;
		}
		
		overNotes = overScore();
		
		types = new int[1];
		types[0] = Scope.INSIDE;
	}

		
	public void setLeft(ScopeDelimiter delimiter){
		leftDelimiter = delimiter;
		leftDelimiter.isAlone = false;
	}

	public void setRight(ScopeDelimiter delimiter){
		rightDelimiter = delimiter;
		rightDelimiter.isAlone = false;
	}
	
	public void removeDependantGestures(){//System.err.println(" &&& " + this);
		rightDelimiter.removeDependantGestures();
		leftDelimiter.removeDependantGestures();
	}
	
	public boolean isPaired(){
		return leftDelimiter != null;
	}
	
	// return the last in time stroke...
	public GeneralInkStroke getFirstStroke(){
		if(leftDelimiter.getFirstStroke().getFirstTimestamp() < rightDelimiter.getFirstStroke().getFirstTimestamp())
		return leftDelimiter.getFirstStroke();
		else return rightDelimiter.getFirstStroke();
	}
	
	public GeneralInkStroke getLastStroke(){
		if(leftDelimiter.getFirstStroke().getFirstTimestamp() < rightDelimiter.getFirstStroke().getFirstTimestamp())
			return rightDelimiter.getFirstStroke();
			else return leftDelimiter.getFirstStroke();
	}
	
	public ScopeDelimiter getLeft(){
		return leftDelimiter;
	}
	
	public ScopeDelimiter getRight(){
		return rightDelimiter;
	}

	private boolean overScore() {
		SingleStrokeAnalysisResult result = leftDelimiter.getFirstStroke().getAnalysisResult();
		if(result.intersects()) return true;
		
		result = rightDelimiter.getFirstStroke().getAnalysisResult();
		if(result.intersects()) return true;
		else return false;
	}

	public boolean intersectsWithScore() {
		return overNotes;
	}
	
	public Shape getEnclosedArea() { 
		double x1 = leftDelimiter.getFirstStroke().getShape().getBounds().getMinX();
		double y1 = leftDelimiter.getFirstStroke().getShape().getBounds().getMinY();
		
		double x2 = rightDelimiter.getFirstStroke().getShape().getBounds().getMaxX();
		double y2 = rightDelimiter.getFirstStroke().getShape().getBounds().getMaxY();
		
		return new Rectangle2D.Double(x1, y1, x2 - x1, y2 - y1);
	}
	
	public boolean isLeaf(){
		return false;
	}	
	
	public Collection<Token> getSubTokens(){
		Vector<Token> subTokens = new Vector<Token>();
		subTokens.add(leftDelimiter);
		subTokens.add(rightDelimiter);
		
		return subTokens;
	}
	
	public Scope getScope(int type){
		if(type!= Scope.INSIDE) return null;
		
		SortedSet<Voice> voices = new TreeSet<Voice>();
		
		SingleStrokeAnalysisResult analysisResult1 = 
			getLeft().getFirstStroke().getAnalysisResult();	
		SingleStrokeAnalysisResult analysisResult2 = 
			getRight().getFirstStroke().getAnalysisResult();	
		
		PolyStaff staff1 = analysisResult1.getClosestPolyStaff();
		PolyStaff staff2 = analysisResult2.getClosestPolyStaff();
		
		for(VoiceLine vline:analysisResult1.getCloseVoiceLines())
			if(vline.getPolyStaff() == staff1) voices.add(vline.getVoice());
		for(VoiceLine vline:analysisResult2.getCloseVoiceLines())
			if(vline.getPolyStaff() == staff2) voices.add(vline.getVoice());

		ScopeInstance scope = new ScopeInstance(voices, 
				leftDelimiter.getStartPointer(), rightDelimiter.getEndPointer(),
			 leftDelimiter.getLeftOuterSpace(), rightDelimiter.getRightOuterSpace());
		
		return new BoundedScope(scope, getCanvases(), type);
	}
	
	public boolean intersects(Line2D line) {
	
		if(leftDelimiter.intersects(line) || rightDelimiter.intersects(line)) return true;
		
		return false;
	}
	
	public Token intersectsWith(Token token) {
		if(leftDelimiter.intersectsWith(token)!=null) return leftDelimiter; 
		if(rightDelimiter.intersectsWith(token)!=null) return rightDelimiter;
		
		return null;
	}	
	
	public String toString(){		
		return "Block";
	}
}
