package inria.paperoles.grammar;

import inria.paperoles.gestures.ComplexInkStroke;
import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.ui.GlobalParameters;

import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;

import papertoolkit.pen.PenSample;

public class GroupingGesture extends ComplexGesture {

	ComplexInkStroke stroke;
	
	boolean isFree = true;
	
	public GroupingGesture(Token token, Group group){
		super(token.getFirstStroke(), token.getCanvas());
		
		gcanvas = token.gcanvas;
		addToken(token); 
		addToken(group);
		
		types = new int[1];
		types[0] = Scope.STAFF_CLOSEST;
	}
	
	public Scope getScope(){
		if(isFree) return activescope;
		else return null;
	}	
	
	public AlternativeScopes getAlternativeScopes(){
		if(isFree) return scopes;
		else return new AlternativeScopes();
	}	
	
	public void setFree(boolean free){
		isFree = free;
	}
	
	public boolean isFree(){
		return isFree;
	}
	
	public Token getGrouper(){
		return tokens.firstElement();
	}
	
	public Group getGroup(){
		return (Group)tokens.lastElement();
	}
	
	public void removeDependantGestures(){//System.err.println(" &&& " + this);
		getGrouper().removeDependantGestures();
	}
	
	public boolean intersects(Line2D line) {
		if(tokens.firstElement().intersects(line)) return true;
		
		return false;
	}
	
	public double getMinX(){
		return tokens.firstElement().stroke.getMinX();
	}
	
	public double getMaxY(){
		return tokens.firstElement().stroke.getMaxY();
	}
	
	public Token intersectsWith(Token token) {
		if(token.gcanvas == gcanvas && tokens.firstElement().intersectsWith(token)!=null) return this;
		
		return null;
	}	
	
	public Token getIntersectedToken(Collection<Token> tokens) { 
		for(Token token:tokens){
			if(token == this || token.gcanvas != gcanvas) continue;
			
			Token t;
			if((t = token.intersectsWith(getGrouper()))!=null) return t;
		}
		
		return null;
	}
	
	
	public Token getProximityToken(Collection<Token> tokens) { 
		double mindist = 5000;
		Token t = null;
		
		for(Token token:tokens){
			if((token instanceof ConnectorToken) || (token instanceof SelectorToken)) continue;
			if(token == this || token.gcanvas != gcanvas) continue;
			
			double dist = token.distanceFrom(getGrouper());
			
			if(dist < mindist){
				mindist = dist;
				t = token;
			}
		}
		
	
		if(mindist < GlobalParameters.MIN_PARAMETER_DISTANCE){
			return t;
		}
		else return null;
	}
	
	public GeneralInkStroke getInsideStroke(){
		ComplexInkStroke stroke = new ComplexInkStroke();
		stroke.addStroke(getGroup().combineStroke());
		
		return stroke;
	}
	
	public GeneralInkStroke getMixedStroke(){
		if(stroke == null){
			stroke = new ComplexInkStroke();
		
			stroke.addStroke(getGroup().combineStroke());
			stroke.addStroke(getGrouper().getFirstStroke());
		}
		
		return stroke;
	}

	public String toString(){
		return "Text";
	}

	////////
	public String getCurvePoints(){
		SingleStrokeAnalysisResult result = getGrouper().stroke.getAnalysisResult();
		PolyStaff staff = result.getClosestPolyStaff();
			
		Object[] samples = Token.filterPoints(getInsideStroke().getSamples());
		
		String s = "";
		s+="( ";
		
		for(int i = 0; i<samples.length; ++i){
			s += (getTime(((PenSample)samples[i]).getX(), staff) + " ");
		}
		s+=") ( ";
		for(int i = 0; i<samples.length; ++i){
			s += (getAmplitude(((PenSample)samples[i]).getY(), staff) + " ");
		}
		s+=")";
		
		return "(" + s + ")";
	}
	
	
	public int getTimePosition(){
		SingleStrokeAnalysisResult result = getGrouper().stroke.getAnalysisResult();
		PolyStaff staff = result.getClosestPolyStaff();
		
		double x_min = 10000000;
		for(PenSample sample:getGrouper().stroke.getSamples()){
			if(sample.getX() < x_min) x_min = sample.getX();
		}
		
		return getTime(x_min, staff);
	}
	
	public String getXRange(){
		SingleStrokeAnalysisResult result = getGrouper().stroke.getAnalysisResult();
		PolyStaff staff = result.getClosestPolyStaff();
		
		double x_min = 10000000, x_max = 0;
		for(PenSample sample:getGrouper().stroke.getSamples()){
			if(sample.getX() < x_min) x_min = sample.getX();
			if(sample.getX() > x_max) x_max = sample.getX();
		}
		
		return "(" + getTime(x_min, staff) + " " + getTime(x_max, staff) + ")";
	}
	
	public String getStartPoint(){
		SingleStrokeAnalysisResult result = getGrouper().stroke.getAnalysisResult();
		PolyStaff staff = result.getClosestPolyStaff();
		
		PenSample sample = getGrouper().stroke.getSamples().get(0);
		
		return "(" + getTime(sample.getX(), staff) + " " + getAmplitude(sample.getY(), staff) + ")";
	}
	
	public String getCentralPoint(){
		SingleStrokeAnalysisResult result = getGrouper().stroke.getAnalysisResult();
		PolyStaff staff = result.getClosestPolyStaff();
		
		Rectangle2D bounds = getGrouper().stroke.getBounds();
		
		return "(" + getTime(bounds.getCenterX(), staff) + " " + getAmplitude(bounds.getCenterY(), staff) + ")";
	}
	
}
