package inria.paperoles.grammar;

import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.SortedSet;
import java.util.TreeSet;

import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.scoping.ScopeInstance;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.ScoreSkeleton;
import inria.paperoles.scoremodel.Voice;
import inria.paperoles.scoremodel.VoiceLine;
import inria.paperoles.ui.GesturesCanvas;

public class LeftDelimiter extends LineDelimiter{
	public LeftDelimiter(GeneralInkStroke stroke, GesturesCanvas gcanvas) {
		super(stroke, gcanvas);
		// TODO Auto-generated constructor stub
		types = new int[1];
		types[0] = Scope.RIGHT;
	}

	RightDelimiter rightDelimiter;
	
	public void setRight(RightDelimiter delimiter){
		rightDelimiter = delimiter;
	}
	
	public boolean isPaired(){
		return rightDelimiter != null;
	}
	
	public RightDelimiter getRight(){
		return rightDelimiter;
	}
	
	
	public boolean matches(ScopeDelimiter right){
		if(right instanceof LeftDelimiter) return false;
		
		SingleStrokeAnalysisResult analysisResult1 = 
			getFirstStroke().getAnalysisResult();	
		SingleStrokeAnalysisResult analysisResult2 = 
			right.getFirstStroke().getAnalysisResult();	
		
		PolyStaff staff1 = analysisResult1.getClosestPolyStaff();
		PolyStaff staff2 = analysisResult2.getClosestPolyStaff();
		
		if(staff2.compareWith(staff1) < 0 ||
				getFirstStroke().getMinX() >= right.getFirstStroke().getMaxX()) return false;
		else return true;
	}
	
	
	public boolean isPairWith(Token token) {
		if(getRight() == null && token instanceof RightDelimiter){
			rightDelimiter = (RightDelimiter)token;
			((RightDelimiter)token).leftDelimiter = this;
			return true;
		}
		else return false;			
	}
	
	public Area getOuterSpace(){
		PolyStaff poly = stroke.getAnalysisResult().getClosestPolyStaff();
		
		Rectangle2D rect = stroke.getBounds();
		double y = Math.min(rect.getMinY(), poly.getMinY() - 40);
		rect.setRect(rect.getMinX(), y, rect.getWidth(), Math.max(rect.getMaxY(), poly.getMaxY() + 40) - y);
		
		Point2D p1 = stroke.getFirstPoint();
		Point2D p2 = stroke.getLastPoint();
		
		Polygon polygon = new Polygon();
		polygon.addPoint((int)p1.getX(), (int)p1.getY());
		polygon.addPoint((int)p2.getX(), (int)p2.getY());
			
		if(p1.getX() > p2.getX() && p1.getY() < p2.getY()
				|| p1.getX() < p2.getX() && p1.getY() > p2.getY()){
			polygon.addPoint((int)p2.getX(), (int)rect.getMaxY());
			polygon.addPoint((int)p1.getX(), (int)rect.getMaxY());
		}
		else {
			polygon.addPoint((int)p2.getX(), (int)rect.getMinY());
			polygon.addPoint((int)p1.getX(), (int)rect.getMinY());
		}
		
		Area area = new Area(rect);

		Area fill = new Area(stroke.getShape());
		fill.add(new Area(polygon));
		area.subtract(fill);
		
		return area;
	}	
	
	public Scope getScope(int type){
		if(type != Scope.RIGHT) return null;
		
		SingleStrokeAnalysisResult analysisResult = getFirstStroke().getAnalysisResult();
//		if(!analysisResult.intersects()) return null;
		
		SortedSet<Voice> voices = new TreeSet<Voice>();		
		PolyStaff staff = analysisResult.getClosestPolyStaff();
	
		for(VoiceLine vline:analysisResult.getCloseVoiceLines()){ 
			if(vline.getPolyStaff() == staff) voices.add(vline.getVoice());
		}

		ScopeInstance scope = new ScopeInstance(voices, 
				analysisResult.getVerticalStartPointer(staff), staff.getEndPointer(), getOuterSpace(), null);
		
		return new BoundedScope(scope, getCanvases(), type);
	}	
	
	public String toString(){		
		return "Left parenthesis";
	}
}
