package inria.paperoles.grammar;

import inria.paperoles.docmodel.Symbol;
import inria.paperoles.scoping.ExtendedShape;
import inria.paperoles.scoremodel.GestureNote;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.ScorePart;
import inria.paperoles.scoremodel.WaveLine;
import inria.paperoles.ui.GesturesCanvas;

import java.awt.Shape;
import java.util.Collection;
import java.util.Vector;

public class PrintedScope extends Scope {

	Collection<Symbol> symbols;

	
	public PrintedScope(GesturesCanvas canvas, PolyStaff pstaff, int type){
		super(canvas, type);
		this.pstaff = pstaff;
	}
	
	public PrintedScope(GesturesCanvas canvas, PolyStaff pstaff, Shape shape, int type){
		super(canvas, shape, type);
		
		this.pstaff = pstaff;
		if(canvas != null) this.symbols = canvas.getPageSkeleton().select(shape);	
	}
	

	public void addSymbol(Symbol symbol){
		if(symbols == null) symbols = new Vector<Symbol>();
		symbols.add(symbol);
		
		shape = symbol.getBounds();
	}
	
	public Collection<Symbol> getSymbols(){
		return symbols;
	}
	
	public void setActive(boolean active){
		for(Symbol symbol:symbols) symbol.setSelected(active);
	}
	
	public boolean equals(Object o){
		if(!(o instanceof PrintedScope)) return false;
		
		if(this == o || ((PrintedScope)o).shape.getBounds().equals(shape.getBounds())) return true;
		else return false;
	}

	@Override
	public String toStringRepresentation() {
		String s = "";
		
		for(Symbol symbol:symbols)
			if(symbol instanceof WaveLine)
				s+=" (" +((WaveLine)symbol).stringRepresentation() +")";
			else if(symbol instanceof GestureNote) 
				s+=" (" +((GestureNote)symbol).stringRepresentation() +")";
			else s+=" (" +symbol +")";
		
		return s;
	}
}
