package inria.paperoles.grammar;

import java.awt.Polygon;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.SortedSet;
import java.util.TreeSet;

import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.scoping.ScopeInstance;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.Voice;
import inria.paperoles.scoremodel.VoiceLine;
import inria.paperoles.ui.GesturesCanvas;

public class RightDelimiter extends LineDelimiter{
	public RightDelimiter(GeneralInkStroke stroke, GesturesCanvas gcanvas) {
		super(stroke, gcanvas);
		// TODO Auto-generated constructor stub
		types = new int[1];
		types[0] = Scope.LEFT;
	}

	LeftDelimiter leftDelimiter;
	
	public void setLeft(LeftDelimiter delimiter){
		leftDelimiter = delimiter;
	}
	
	public boolean isPaired(){
		return leftDelimiter != null;
	}
	
	public LeftDelimiter getLeft(){
		return leftDelimiter;
	}
	
	public boolean isPairWith(Token token) {
		if(getLeft() == null && token instanceof LeftDelimiter){
			leftDelimiter = (LeftDelimiter)token;
			((LeftDelimiter)token).rightDelimiter = this;
			return true;
		}
		else return false;			
	}
	
	public Area getOuterSpace(){
		PolyStaff poly = stroke.getAnalysisResult().getClosestPolyStaff();
		
		Rectangle2D rect = stroke.getBounds();
		double y = Math.min(rect.getMinY(), poly.getMinY() - 40);
		rect.setRect(rect.getMinX(), y, rect.getWidth(), Math.max(rect.getMaxY(), poly.getMaxY() + 40) - y);		
		
		Point2D p1 = stroke.getFirstPoint();
		Point2D p2 = stroke.getLastPoint();
		
		Polygon polygon = new Polygon();
		polygon.addPoint((int)p1.getX(), (int)p1.getY());
		polygon.addPoint((int)p2.getX(), (int)p2.getY());
			
		if(p1.getX() > p2.getX() && p1.getY() < p2.getY()
				|| p1.getX() < p2.getX() && p1.getY() > p2.getY()){
			polygon.addPoint((int)p2.getX(), (int)rect.getMinY());
			polygon.addPoint((int)p1.getX(), (int)rect.getMinY());
		}
		else {
			polygon.addPoint((int)p2.getX(), (int)rect.getMaxY());
			polygon.addPoint((int)p1.getX(), (int)rect.getMaxY());
		}
		
		Area area = new Area(rect);

		Area fill = new Area(stroke.getShape());
		fill.add(new Area(polygon));
		area.subtract(fill);
		
		return area;
	}
	
	public boolean matches(ScopeDelimiter left){
		if(left instanceof RightDelimiter) return false;
		
		SingleStrokeAnalysisResult analysisResult1 = 
			left.getFirstStroke().getAnalysisResult();	
		SingleStrokeAnalysisResult analysisResult2 = 
			getFirstStroke().getAnalysisResult();	
		
		PolyStaff staff1 = analysisResult1.getClosestPolyStaff();
		PolyStaff staff2 = analysisResult2.getClosestPolyStaff();
		
		if(staff2.compareWith(staff1) < 0 ||
				left.getFirstStroke().getMinX() >= getFirstStroke().getMaxX()) return false;
		else return true;
	}
	
	public Scope getScope(int type){
		if(type != Scope.LEFT) return null;		
		
		SingleStrokeAnalysisResult analysisResult = getFirstStroke().getAnalysisResult();
	//	if(!analysisResult.intersects()) return null;
		
		SortedSet<Voice> voices = new TreeSet<Voice>();
		
		PolyStaff staff = analysisResult.getClosestPolyStaff();
		
		for(VoiceLine vline:analysisResult.getCloseVoiceLines())
			if(vline.getPolyStaff() == staff) voices.add(vline.getVoice());

		ScopeInstance scope = new ScopeInstance(voices, staff.getStartPointer(),
				analysisResult.getVerticalEndPointer(staff), null, getOuterSpace());
		
		return new BoundedScope(scope, getCanvases(), PrintedScope.LEFT);
	}	
	
	public String toString(){		
		return "Right parenthesis";
	}
}
