package inria.paperoles.grammar;

import inria.paperoles.scoping.ExtendedShape;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.ui.GesturesCanvas;

import java.awt.Rectangle;
import java.awt.Shape;

public abstract class Scope implements Comparable{
	
	public final static int NONE = 0;
	public final static int MULTISTAFF_UNDER = 1;
	public final static int MULTISTAFF_OVER = 2;
	public final static int STAFF_CLOSEST = 3;
	public final static int STAFF_UNDER = 4;
	public final static int STAFF_OVER = 5;
	public final static int OVERLAPPING_MEASURE = 6;
	public final static int OVERLAPPING_STAFF = 7;
	public final static int INSIDE = 8;
	public final static int START = 9;
	public final static int END = 10;
	public final static int LEFT = 11;
	public final static int RIGHT = 12;
	public final static int NOTE_LEFT = 13;
	public final static int NOTE_RIGHT = 14;
	public final static int SYMBOLS_INSIDE = 15;
	public final static int POINTER = 16;
	
	public static final int NOTES_UNDER = 17;
	public static final int NOTES_UP = 18;
	public static final int CENTER_UP = 19;
	public static final int CENTER_UNDER = 20;
	public static final int CENTER_CLOSEST = 21;
	public static final int NOTES_CLOSEST = 22;
	
	public static final int CURVE_POINTS = 23;
	
	Shape shape;
	GesturesCanvas canvas;
	int type;
	PolyStaff pstaff;
		
	public Scope(GesturesCanvas canvas, int type){
		this.canvas = canvas;
		this.type = type;
	}
	
	public Scope(GesturesCanvas canvas, Shape shape){
		this.shape = shape;
		this.canvas = canvas;
	}
	
	public Scope(GesturesCanvas canvas, Shape shape, int type){
		this.shape = shape;
		this.canvas = canvas;
		this.type = type;
	}
	
	
	public PolyStaff getPolyStaff(){
		return pstaff;
	}
	
	public int getType(){
		return type;
	}
	
	public Shape getShape(){
		return shape;
	}
	
	public Shape getShape(GesturesCanvas canvas){
		if(canvas == this.canvas) return shape;
		else return null;
	}
	
	public boolean inGesturesCanvas(GesturesCanvas gcanvas){
		return canvas == gcanvas;
	}
	
	static public String getTypeName(int type){
		switch(type){
			case STAFF_CLOSEST:
				return "Closest staff";
			case STAFF_UNDER:
				return "Staff under stroke";
			case STAFF_OVER:
				return "Staff over stroke";
			case MULTISTAFF_UNDER:
				return "Staffs under";
			case MULTISTAFF_OVER:
				return "Staffs over";		
			case NOTES_UNDER:
				return "Edges under";
			case NOTES_UP:
				return "Edges over";
			case NOTES_CLOSEST:
				return "Closest Edges";
			case CENTER_UP:
				return "Chord over";
			case CENTER_UNDER:
				return "Chord under";
			case CENTER_CLOSEST:
				return "Closest chord";
			case POINTER:
				return "Pointer";
			case CURVE_POINTS:
				return "Curve Points";
			case NONE:
				return "None";	
			default:
				return "Score Area";				
		}
	}
	
	public int compareTo(Object s) {
		if(this.equals(s)) return 0;

		if(getArea() > ((Scope)s).getArea()) return -1;
		else return 1;
	}
	
	public double getArea(){
		Rectangle rect = shape.getBounds();
		
		return rect.getWidth()*rect.getHeight();
	}
	
	/*public double getArea(GesturesCanvas canvas){
		Rectangle rect = shape.getBounds();
		
		return rect.getWidth()*rect.getHeight();
	}*/
	
	public String toString(){
		return getTypeName(type);
	}

	abstract public String toStringRepresentation();
}
