package inria.paperoles.grammar;

import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.Vector;

import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.scoping.ScopeInstance;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.ScoreSkeleton;
import inria.paperoles.scoremodel.Voice;
import inria.paperoles.scoremodel.VoiceLine;

public class ScopingPair extends Token{

	boolean overNotes = false;
	
	public ScopingPair(ScopeDelimiter delimiter1, ScopeDelimiter delimiter2) {
		super(null, null);
		
		gcanvases.add(delimiter1.getCanvas());
		gcanvases.add(delimiter2.getCanvas());
		
		delimiter1.setParent(this);
		delimiter2.setParent(this);
		
		if(delimiter1 instanceof LeftDelimiter){
			leftDelimiter = (LeftDelimiter)delimiter1;
			rightDelimiter = (RightDelimiter)delimiter2;
		}
		else {
			leftDelimiter = (LeftDelimiter)delimiter2;
			rightDelimiter = (RightDelimiter)delimiter1;			
		}
		
		overNotes = overScore();
		
		types = new int[1];
		types[0] = Scope.INSIDE;
	}

	LeftDelimiter leftDelimiter;
	RightDelimiter rightDelimiter;
		
	public void setLeft(LeftDelimiter delimiter){
		leftDelimiter = delimiter;
		leftDelimiter.isAlone = false;
	}

	public void setRight(RightDelimiter delimiter){
		rightDelimiter = delimiter;
		rightDelimiter.isAlone = false;
	}
	
	public boolean isPaired(){
		return leftDelimiter != null;
	}
	
	// return the last in time stroke...
	public GeneralInkStroke getFirstStroke(){
		if(leftDelimiter.getFirstStroke().getFirstTimestamp() < rightDelimiter.getFirstStroke().getFirstTimestamp())
		return leftDelimiter.getFirstStroke();
		else return rightDelimiter.getFirstStroke();
	}
	
	public GeneralInkStroke getLastStroke(){
		if(leftDelimiter.getFirstStroke().getFirstTimestamp() < rightDelimiter.getFirstStroke().getFirstTimestamp())
			return rightDelimiter.getFirstStroke();
			else return leftDelimiter.getFirstStroke();
	}
	
	public LeftDelimiter getLeft(){
		return leftDelimiter;
	}
	
	public RightDelimiter getRight(){
		return rightDelimiter;
	}

	private boolean overScore() {
		SingleStrokeAnalysisResult result = leftDelimiter.getFirstStroke().getAnalysisResult();
		if(result.intersects()) return true;
		
		result = rightDelimiter.getFirstStroke().getAnalysisResult();
		if(result.intersects()) return true;
		else return false;
	}

	public boolean intersectsWithScore() {
		return overNotes;
	}
	
	public Shape getEnclosedArea() { 
		double x1 = leftDelimiter.getFirstStroke().getShape().getBounds().getMinX();
		double y1 = leftDelimiter.getFirstStroke().getShape().getBounds().getMinY();
		
		double x2 = rightDelimiter.getFirstStroke().getShape().getBounds().getMaxX();
		double y2 = rightDelimiter.getFirstStroke().getShape().getBounds().getMaxY();
		
		return new Rectangle2D.Double(x1, y1, x2 - x1, y2 - y1);
	}
	
	public boolean isLeaf(){
		return false;
	}	
	
	public Collection<Token> getSubTokens(){
		Vector<Token> subTokens = new Vector<Token>();
		subTokens.add(leftDelimiter);
		subTokens.add(rightDelimiter);
		
		return subTokens;
	}
	
	public Scope getScope(int type){
		if(type!= Scope.INSIDE) return null;
		
		SortedSet<Voice> voices = new TreeSet<Voice>();
		
		SingleStrokeAnalysisResult analysisResult1 = 
			getLeft().getFirstStroke().getAnalysisResult();	
		SingleStrokeAnalysisResult analysisResult2 = 
			getRight().getFirstStroke().getAnalysisResult();	
		
		PolyStaff staff1 = analysisResult1.getClosestPolyStaff();
		PolyStaff staff2 = analysisResult2.getClosestPolyStaff();
		
		for(VoiceLine vline:analysisResult1.getCloseVoiceLines())
			if(vline.getPolyStaff() == staff1) voices.add(vline.getVoice());
		for(VoiceLine vline:analysisResult2.getCloseVoiceLines())
			if(vline.getPolyStaff() == staff2) voices.add(vline.getVoice());

		ScopeInstance scope = new ScopeInstance(voices, 
				analysisResult1.getVerticalStartPointer(analysisResult1.getClosestPolyStaff()), 
				analysisResult2.getVerticalEndPointer(analysisResult2.getClosestPolyStaff()),
			 leftDelimiter.getOuterSpace(), rightDelimiter.getOuterSpace());
		
		return new BoundedScope(scope, getCanvases(),type);
	}
	
	public boolean intersects(Line2D line) {
	
		if(leftDelimiter.intersects(line) || rightDelimiter.intersects(line)) return true;
		
		return false;
	}
}
