package inria.paperoles.io;

import inria.paperoles.scoremodel.Measure;
import inria.paperoles.scoremodel.MultiplePageSkeleton;
import inria.paperoles.scoremodel.Note;
import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.ScoreSkeleton;
import inria.paperoles.scoremodel.Segment;
import inria.paperoles.scoremodel.VoiceLine;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

public class Transformer {
	
	public final static String PATH = "descr/";
	
	ScoreSkeleton skeleton;	
	String notesFilePath;
	
	String tfile;
	
	public Transformer(String filename){
		notesFilePath = PATH + filename;	
		
		tfile = notesFilePath + "2";
	}
	
	FileWriter output;
	
	public MultiplePageSkeleton build() throws NumberFormatException, IOException {
		MultiplePageSkeleton multiskeleton = new MultiplePageSkeleton();
		BufferedReader input = new BufferedReader(new FileReader(notesFilePath));
		output = new FileWriter(tfile);
		
		String source = getMusicFile(input);

		
	//	int pageNum = getPageNum(input); 
		for(ScoreSkeleton pageSkeleton = getNextPage(input, multiskeleton); pageSkeleton != null; pageSkeleton = getNextPage(input, multiskeleton)){
			multiskeleton.addPageSkeleton(pageSkeleton);
	
		}
		
		output.close();
		return multiskeleton;
	}
	
	/// Normally, i should first call buildSkeleton and then readNotes.
	// in order to build the model correctly...
	int polyStaffNum = 1;
	private ScoreSkeleton getNextPage(BufferedReader input, 
			MultiplePageSkeleton multiskeleton) throws NumberFormatException, IOException {

		if(entry == null){ 
			String line = input.readLine();
		

			
			if(line == null) return null;
			output.write(line + "\n");
			entry = new InputEntry(line);
		}
			
		if(!entry.isPage()) return null;	
		
		skeleton = new ScoreSkeleton(multiskeleton);
		entry = null;
		
		for(PolyStaff polyStaff = getNextPolyStaff(input, skeleton, polyStaffNum); polyStaff != null; polyStaff = getNextPolyStaff(input, skeleton, ++polyStaffNum))
			skeleton.addDocPart(polyStaff);
		
		return skeleton;
	}

	InputEntry entry;
	private PolyStaff getNextPolyStaff(BufferedReader input, ScoreSkeleton skeleton, int num) throws IOException {
		if(entry == null){
			String line = input.readLine();
		
			
			if(line == null) return null;
			output.write(line + "\n");
			entry = new InputEntry(line); 
		}
		
		if(!entry.isMultiStaff()) return null;
		
System.err.println("polystaff");				
		PolyStaff polyStaff = new PolyStaff(skeleton, num, entry.getX(), entry.getY(), 
				entry.getWidth(), entry.getHeight());
	
		entry = null;
		
		int i = 1;
		for(VoiceLine staff = getNextStaff(input, polyStaff, i); staff != null; staff = getNextStaff(input, polyStaff, ++i)){
			//polyStaff.addVoiceLine(staff);
			skeleton.addDocPart(staff);
		}
		
		return polyStaff;
	}


	private VoiceLine getNextStaff(BufferedReader input, PolyStaff polyStaff, int num) throws IOException {
		if(entry == null){
			String line = input.readLine();
			
			if(line == null) return null;
			output.write(line + "\n");
			entry = new InputEntry(line);
		}
		
		if(!entry.isStaff()) return null;
		
System.err.println("staff");		
		VoiceLine staff = new VoiceLine(polyStaff, num, entry.getX(), entry.getY(), 
				entry.getWidth(), entry.getHeight());
			
		entry = null;
		
		int i = 0;
		for(Segment segment = getNextSegment(input, staff, i); segment != null; segment = getNextSegment(input, staff, ++i)){
			//staff.addSegment(measure);
		}
		
		return staff;
	}

	
	private Segment getNextSegment(BufferedReader input, VoiceLine staff, int num) throws IOException {		
		if(entry == null){
			String line = input.readLine();
				
			if(line == null) return null;
			
			output.write(line + "\n");
			entry = new InputEntry(line);
		}
		
		if(!entry.isMeasure()) return null;

System.err.println("measure");			
		Measure measure = staff.getPolyStaff().getMeasureAt(num);
		if(measure == null) measure = new Measure();
		Segment segment = new Segment();
		
		int y1 = (int)Math.min(staff.getMinY(), entry.getY());
		int y2 = (int)Math.max(staff.getMaxY(), entry.getY() + entry.getHeight());

		if(measure.getPolyStaff() == null){
			measure.initialize(staff.getPolyStaff(), entry.getX(), entry.getX() + entry.getWidth() - 1);
			skeleton.addDocPart(measure);
		}
		segment.initialize(measure, staff, entry.getX(), y1, entry.getWidth(), y2-y1);
		skeleton.addDocPart(segment);

		entry = null;
		
		for(Note note = getNextNote(input, segment); note != null; note = getNextNote(input, segment))
			skeleton.addSymbol(note);
		
		
		return segment;
	}
	
String oldline;	
	private Note getNextNote(BufferedReader input, Segment segment) throws IOException {
		
		String line = null;
		if(entry == null){
					
			line = input.readLine();

			if(line == null) return null;
		if(line.equals(oldline)) return getNextNote(input, segment);				
		oldline = line;				
			
			entry = new InputEntry(line);
		}
		
		if(!entry.isNote()){
			if(line!=null) output.write(line + "\n");
			return null;
		}

		
		output.write("(group " + entry.getX() +" " + entry.getY() 
				+ " " + entry.getWidth() + " "+ entry.getHeight() + ")\n");
		output.write("(chord " + entry.getX() +" " + entry.getY() 
				+ " " + entry.getWidth() + " "+ entry.getHeight() + ")\n");
		output.write(line + "\n");
		
System.err.println("note");		
	

		Note note = new Note( entry.getX(), entry.getY());
			
		entry = null;
		
		return note;
	}

	private String getMusicFile(BufferedReader input) throws IOException {
		String line = input.readLine();
		
		output.write(line + "\n");
		
		String[] par = getParameters(line);
		if(par.length < 2) throw new IOException("wrong number of arguments");
		if(!par[0].equals("score")) throw new IOException("expected 'score' as first argument");
		
		return par[1];
	}

	private String[] getParameters(String line){
		return line.substring(1, line.length() - 1).split(" ");
	}
	
	class InputEntry{
		static final short ERROR = -1;
		static final short PAGE = 0;
		static final short MULTISTAFF = 1;
		static final short STAFF = 2;
		static final short MEASURE = 3;
		static final short NOTE = 4;
		
		final static double scale = 1;//.48 
		
		int type = -1;
		
		int x, y, w, h;
		
		InputEntry(String line){
			String []par = getParameters(line);
	
			if(par.length >= 1 && par[0].equalsIgnoreCase("page")){	
				type = PAGE;
				return;
			}
			else if(par.length != 5){
				type = ERROR; 
				return;
			}
			
			x = (int)(scale*(new Integer(par[1])).intValue());
			y = (int)(scale*(new Integer(par[2])).intValue()) /*- 10*/;
			w = (int)(scale*(new Integer(par[3])).intValue());
			h = (int)(scale*(new Integer(par[4])).intValue());
			
			if(par[0].equalsIgnoreCase("multistaff")) type = MULTISTAFF;
			else if(par[0].equalsIgnoreCase("staff")) type = STAFF;
			else if(par[0].equalsIgnoreCase("measure")) type = MEASURE;
			else if(par[0].equalsIgnoreCase("note")) type = NOTE;
			else type = ERROR;
			
		//	if(type == NOTE) y+=10;		
		}
		
		
		int getX(){
			return x;
		}
		
		int getY(){
			return y;
		}
		
		int getWidth(){
			return w;
		}
		
		int getHeight(){
			return h;
		}

			
		boolean isStaff(){
			return type == STAFF;
		}

		boolean isMultiStaff(){
			return type == MULTISTAFF;
		}
		
		boolean isNote(){
			return type == NOTE;
		}
		
		boolean isMeasure(){
			return type == MEASURE;
		}
		
		boolean isPage(){
			return type == PAGE;
		}
		
		public String toString(){
			return ""+ type;
		}
	}
	
}
