package inria.paperoles.scoping;

import inria.paperoles.scoremodel.PolyStaff;
import inria.paperoles.scoremodel.ScoreSkeleton;
import inria.paperoles.scoremodel.ScoreVerticalPointer;
import inria.paperoles.scoremodel.Voice;
import inria.paperoles.scoremodel.VoiceLine;

import java.awt.Shape;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.Vector;

public class ScopeInstance {

	SortedSet<Voice> voices;
	ScoreVerticalPointer pstart, pend;
	
	ExtendedShape shape;
	
	
	public ScopeInstance(ExtendedShape shape){
		this.shape = shape;
	}
	
	
	public ScopeInstance(VoiceLine voiceLine, ScoreVerticalPointer pstart, ScoreVerticalPointer pend){
		this.pstart = pstart;
		this.pend = pend;
		ScoreSkeleton skeleton = (pstart != null)?pstart.getSkeleton():pend.getSkeleton();
		
		voices = new TreeSet<Voice>();
		voices.add(voiceLine.getVoice());
		
		shape = new ExtendedShape();
		shape.put(skeleton, VoiceLine.getAreaBetweenPoints(voiceLine, voiceLine,
				pstart, pend));
				/*voiceLine.getAreaBetweenPoints(pstart, pend)*/
	}
	
	public ScopeInstance(ScoreVerticalPointer pstart, ScoreVerticalPointer pend){
		this(pstart, pend, pstart.getPolyStaff().getVoiceLines());
	}
	
	
	public ScopeInstance(ScoreVerticalPointer pstart, ScoreVerticalPointer pend,
			Vector<VoiceLine> voiceLines){
		this.pstart = pstart;
		this.pend = pend;
		ScoreSkeleton skeleton = (pstart != null)?pstart.getSkeleton():pend.getSkeleton();
		
		voices = new TreeSet<Voice>();
		for(VoiceLine vline:voiceLines)
			voices.add(vline.getVoice());
		
		shape = new ExtendedShape();
		shape.put(skeleton, VoiceLine.getAreaBetweenPoints(voiceLines.firstElement(), voiceLines.lastElement(), pstart, pend));
	}
	
	
	// This implementation does not consider more flexible scope instances...
	// ok... i will change it soon....
	public ScopeInstance(SortedSet<Voice> voices, ScoreVerticalPointer pstart, ScoreVerticalPointer pend,
			Area leftArea, Area rightArea){
		this.pstart = pstart;
		this.pend = pend;
		
		this.voices = voices; 
		
		shape = new ExtendedShape();
		////////////////////////////////////////
		// and compute now the associated shape
		if(pstart == null || pend == null || pstart.getPolyStaff() == pend.getPolyStaff()){
			PolyStaff pstaff = (pstart != null)?pstart.getPolyStaff():pend.getPolyStaff();
			
			Area area = new Area(VoiceLine.getAreaBetweenPoints(
					voices.first().getVoiceLine(pstaff), voices.last().getVoiceLine(pstaff), pstart, pend));
			
			if(leftArea != null) area.subtract(leftArea);
			if(rightArea != null) area.subtract(rightArea);
			
			shape.put(pstart!=null?pstart.getSkeleton():pend.getSkeleton(), area);
		}
		else {
			Shape tmpshape = VoiceLine.getAreaBetweenPoints(voices.first().getVoiceLine(pstart.getPolyStaff()), 
					voices.last().getVoiceLine(pstart.getPolyStaff()), pstart, null);
					
			ScoreSkeleton skeleton = pstart.getSkeleton();
			Area area = new Area(tmpshape);
			if(leftArea != null) area.subtract(leftArea);
			
			for(PolyStaff pstaff = pstart.getPolyStaff().getNext(); 
				pstaff != pend.getPolyStaff(); pstaff = pstaff.getNext()){

				tmpshape = VoiceLine.getAreaBetweenPoints(voices.first().getVoiceLine(pstaff), 
						voices.last().getVoiceLine(pstaff), null, null);
				
				if(skeleton != pstaff.getSkeleton()){
					shape.put(skeleton, area);
					
					area = new Area(tmpshape);
					skeleton = pstaff.getSkeleton();
				}
				else area.add(new Area(tmpshape));
			}
			
			PolyStaff pstaff = pend.getPolyStaff();
			tmpshape = VoiceLine.getAreaBetweenPoints(voices.first().getVoiceLine(pstaff), 
					voices.last().getVoiceLine(pstaff), null, pend);
			
			if(skeleton != pstaff.getSkeleton()){
				shape.put(skeleton, area);
				area = new Area(tmpshape);
			}
			else area.add(new Area(tmpshape));
			
	
			if(rightArea != null) area.subtract(rightArea);
			
			shape.put(pstaff.getSkeleton(), area);
		}
	}
	
	
	public ExtendedShape getScope(){
		return shape;
	}
	
	public SortedSet<Voice> getVoices(){
		return voices;
	}
	
	public ScoreVerticalPointer getStartPointer(){
		return pstart;
	}
	
	public ScoreVerticalPointer getEndPointer(){
		return pend;
	}
	
}
