package inria.paperoles.scoremodel;

import java.awt.Color;
import java.awt.Shape;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.Vector;

import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;

import inria.paperoles.docmodel.DocPart;
import inria.paperoles.docmodel.Symbol;

public class VoiceLine extends ScorePart{
	
	PolyStaff polyStaff;
	final static Color STROKE_COLOR = new Color(200, 200, 170);
	final static Color FILL_COLOR = new Color(240, 240, 200);
	
	Vector<Segment> segments;
	Voice voice;
	int num;
	
	/*
	public VoiceLine(PolyStaff parent, int id, int y, int height){
		super((int)parent.getX(), y, (int)parent.getWidth(), height);
		
		setStrokePaint(STROKE_COLOR);
		setPaint(FILL_COLOR);
		
		polyStaff = parent;
		parent.addVoiceLine(this);
		
		Voice voice = Voice.getVoice(id);
		if(voice == null) voice = new Voice(id);
		
		segments = new Vector<Segment>();
		voice.addVoiceLine(this);
		num = voice.size();
		
		this.voice = voice;
	}*/
	
	
	public VoiceLine(PolyStaff parent, int id, int x, int y, int width, int height){
		super(x, y, width, height);

		
		setStrokePaint(STROKE_COLOR);
		setPaint(FILL_COLOR);
		
		polyStaff = parent;
		parent.addVoiceLine(this);
		
		Voice voice = Voice.getVoice(id);
		if(voice == null) voice = new Voice(id);
		
		segments = new Vector<Segment>();
		voice.addVoiceLine(this);
		
		this.voice = voice;
	}
	
	public static Shape getAreaBetweenPoints(VoiceLine vline1, VoiceLine vline2, ScoreVerticalPointer start, ScoreVerticalPointer end){
		double ymin = 5000, ymax=0, xmin, xmax;
		int index1, index2;

		if(start == null){
			xmin = vline1.getX();
			index1 = 0;
		}
		else{
			xmin = Math.max(vline1.getX(), start.getX());
			index1 = vline1.segments.indexOf(start.getMeasure().getSegment(vline1.voice));		
		}
	
		if(end == null){
			xmax = vline1.getMaxX();
			index2 = vline1.segments.size() - 1;
		}
		else {
			xmax = Math.min(vline1.getMaxX(), end.getX());
			index2 = vline1.segments.indexOf(end.getMeasure().getSegment(vline1.voice));
		}
			
		for(int i=index1; i<=index2; ++i){
			Segment seg1 = vline1.segments.elementAt(i);
			Segment seg2 = vline2.segments.elementAt(i);
			if(seg1.getMinY() < ymin) ymin = seg1.getMinY();
			if(seg2.getMaxY() > ymax) ymax = seg2.getMaxY();
		}
			
		return new Rectangle2D.Double(xmin, ymin, xmax - xmin, ymax - ymin);
	}
	
	/*
	public Shape getAreaBetweenPoints(ScoreVerticalPointer start, ScoreVerticalPointer end){
		double ymin = 5000, ymax=0, xmin, xmax;
		int index1, index2;

		if(start == null){
			xmin = getX();
			index1 = 0;
		}
		else{
			xmin = Math.max(getX(), start.getX());
			index1 = segments.indexOf(start.getMeasure().getSegment(voice));
		}
	
		if(end == null){
			xmax = getMaxX();
			index2 = segments.size() - 1;
		}
		else {
			xmax = Math.max(getMaxX(), end.getX());
			index2 = segments.indexOf(end.getMeasure().getSegment(voice));
		}
			
		for(int i=index1; i<=index2; ++i){
			Segment seg = segments.elementAt(i);
			if(seg.getMinY() < ymin) ymin = seg.getMinY();
			if(seg.getMaxY() > ymax) ymax = seg.getMaxY();
		}
			
		return new Rectangle2D.Double(xmin, ymin, xmax - xmin, ymax - ymin);
	}
	*/
	
	public PolyStaff getPolyStaff(){
		return polyStaff;
	}
	
	public Voice getVoice(){
		return voice;
	}
	
	public VoiceLine getPreviousVoiceLine(){
		if(isFirstLineInStaff()){
			PolyStaff pstaff = polyStaff.getPreviousInPage();
			if(pstaff == null) return null;
			else return pstaff.voiceLines.lastElement();
		}
		else return polyStaff.voiceLines.get(polyStaff.voiceLines.indexOf(this) - 1);
	}
	
	public VoiceLine getNextVoiceLine(){
		if(isLastLineInStaff()){
			PolyStaff pstaff = polyStaff.getNextInPage();
			if(pstaff == null) return null;
			else return pstaff.voiceLines.firstElement();
		}
		else return polyStaff.voiceLines.get(polyStaff.voiceLines.indexOf(this) + 1);
	}
	
	public VoiceLine getNextLineInVoice(){
		int index = voice.indexOf(this) + 1;
		if(index == voice.size()) return null;
		else return voice.get(index);
	}
	
	public boolean isLastLineInStaff(){
		return polyStaff.voiceLines.indexOf(this) == polyStaff.voiceLines.size() - 1;
	}
	
	public boolean isFirstLineInStaff(){
		return polyStaff.voiceLines.indexOf(this) == 0;
	}
	
	public Collection<VoiceLine> getNextLinesInVoice(){
		return voice.getLinesFrom(this);
	}

	public Collection<VoiceLine> getPreviousLinesInVoice(){
		return voice.getLinesTo(this);
	}	
	
	public void addSegment(Segment segment) {
		segments.add(segment);
	}
	
	public Collection<Segment> getSegments(){
		return segments;
	}
	
	public ScorePart getScorePartAt(double x) {
		for(Segment seg:segments){
			if(seg.getMaxX() > x)
				for(Chord chord:seg.chords){
					if(chord.center_x + 6 >= x)
						return chord;
				}
		}
		
		VoiceLine line = getNextLineInVoice();
		if(line == null) return new EndScore(segments.lastElement());
		return line.segments.firstElement();
	}
	
	public Measure getMeasureAt(double x){
		for(Segment seg:segments){
			if(seg.getMinX() <= x && seg.getMaxX()>=x)
				return seg.getMeasure();
		}
		return null;
	}

	public Symbol getSymbolAt(Point2D headPoint) {
		double mindistance = 99999999;
		
		Chord mchord = null;
		for(Segment seg:segments){
			for(Chord chord:seg.chords){
				double dist = chord.getBounds().getCenter2D().distanceSq(headPoint);
				
				if(dist < mindistance){
					mindistance = dist;
					mchord = chord;
				}
			}
		}
		
		if(mchord == null) return null;
			
		mindistance = 999999999;
		Note minnote = null;
		for(Note note:mchord.getNotes()){
			double dist = note.getBounds().getCenter2D().distanceSq(headPoint);
			
			if(dist < mindistance){
				mindistance = dist;
				minnote = note;
			}
		}
					
		return minnote;
	}
	
}
