package inria.paperoles.ui;

import inria.paperoles.data.PenDataEntry;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class DataList extends JList implements ListSelectionListener{
	
	HashSet<String> dates;
	final static Color DATE_COLOR = new Color(210, 210, 230);
	
	DataListWindow parent;
	
	public DataList(DataListWindow parent){
		super(new DefaultListModel());
		
		setCellRenderer(new OperationCellRenderer());
		setVisibleRowCount(-1);
		setLayoutOrientation(JList.VERTICAL);
		
		addListSelectionListener(this);
	/*	addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e) {
				if(e.getClickCount() < 2) return;
				
			}
		});*/
		
		dates = new HashSet<String>();
		
		this.parent = parent;
	}
	
	public void addEntry(PenDataEntry entry, boolean assigned){

		ListItem listItem;
		
		if(!dates.contains(entry.getDate())){
			listItem = new ListItem(entry, true, false);
			((DefaultListModel)getModel()).insertElementAt(listItem, 0);
			
			dates.add(entry.getDate());
		}
		
		listItem = new ListItem(entry, false, assigned);
		((DefaultListModel)getModel()).insertElementAt(listItem, 1);

	}
	
	int []shownIndices;
	public void showSelections(){
		if(shownIndices!=null)
			for(int i=0; i<shownIndices.length; ++i)
				((ListItem)((DefaultListModel)getModel()).get(shownIndices[i])).setShow(false);
		
		shownIndices = getSelectedIndices();
		for(int i=0; i<shownIndices.length; ++i) {	
			ListItem listItem = (ListItem)((DefaultListModel)getModel()).get(shownIndices[i]);
			listItem.setShow(true);

			parent.dispatchData(GlobalParameters.DATA_FOLDER + listItem.getEntry().getFileName());
		}
			
		repaint();
	}
	
	public void assignSelections(String source) {
		shownIndices = getSelectedIndices();
		for(int i=0; i<shownIndices.length; ++i)		
			((ListItem)((DefaultListModel)getModel()).get(shownIndices[i])).setAssign(source);
		
		parent.getAssignButton().setEnabled(false);
		parent.getUnAssignButton().setEnabled(true);
		
		repaint();
	}
	
	public void unassignSelections() {
		shownIndices = getSelectedIndices();
		for(int i=0; i<shownIndices.length; ++i)		
			((ListItem)((DefaultListModel)getModel()).get(shownIndices[i])).setAssign(null);
		
		parent.getUnAssignButton().setEnabled(false);
		parent.getAssignButton().setEnabled(true);
		
		repaint();
	}
	
	final static Color selectColor = GlobalParameters.LIST_SELECTION_COLOR;
		
	private class OperationCellRenderer extends JLabel implements ListCellRenderer{

		public OperationCellRenderer(){
			super();
			setFont(new Font("Arial", Font.PLAIN, 10));
			setForeground(Color.black);
			setOpaque(true);
	       	setVerticalTextPosition(JLabel.BOTTOM);
        	setHorizontalTextPosition(JLabel.CENTER);
    		setBorder(BorderFactory.createEmptyBorder(3,3,3,3));
		}

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
        {
        	ListItem item = (ListItem)value;

            if(item.dateFlag) {
            	setText(item.getDate());
            	setBackground(DATE_COLOR);
            	setForeground(Color.black);
            }
            else{
            	if(item.assigned) setText(item.getLabel() + " *");
            	else setText(item.getLabel());
            		
            	setForeground(item.isShown()? Color.blue:Color.black);
            	setBackground(isSelected ?  selectColor: Color.white);
            }
           
            return this;
        }
	}
	
    private class ListItem
    {
    	
    	PenDataEntry entry;
        boolean dateFlag;
        boolean shown = false;
        boolean assigned = false;
    	
        ListItem(PenDataEntry entry, boolean dateFlag, boolean assigned)
        {
            this.entry = entry;
            this.dateFlag = dateFlag;
            this.assigned = assigned;
        }
  
        public void setShow(boolean b) {
        	if(dateFlag) return;
        		
			shown = b;
		}

        public PenDataEntry getEntry(){
        	return entry;
        }
        
        public void setAssign(String source){
           	if(dateFlag) return;
    		
           	entry.linkWithScoreFile(source);
			assigned = (source!=null);
        }
        
        public void match(String source){
        	if(dateFlag) return;
        	
        	if(source!=null &&  entry.getScoreFile()!=null
        		&&entry.getScoreFile().equals(source)) assigned = true;
        	else assigned = false;
        }
        
		public String getDate(){
        	return entry.getDate();
        }
        
        public boolean isShown(){
        	return shown;
        }
        
        public String getLabel(){
        	return "          " + entry.getTime();
        }
        
        public String toString(){
        	return dateFlag + "    " + entry.getTime() + " " + entry.getDate();
        }
    }

	public void valueChanged(ListSelectionEvent e) {
		
		if(!parent.scoreExists()) return;
		
		int[] shownIndices = getSelectedIndices();
		
		ListItem item;
		boolean toAssign = false;
		boolean toUnAssign = false;
		boolean toShow = false;
		
		for(int i=0; i<shownIndices.length; ++i){	
			item = ((ListItem)((DefaultListModel)getModel()).get(shownIndices[i]));
			if(!item.dateFlag && !item.assigned) toAssign = true;
			if(!item.dateFlag && item.assigned) toUnAssign = true;
			if(!item.dateFlag) toShow = true;
		}
		
		if(toAssign) parent.getAssignButton().setEnabled(true);
		else parent.getAssignButton().setEnabled(false);
		
		if(toUnAssign) parent.getUnAssignButton().setEnabled(true);
		else parent.getUnAssignButton().setEnabled(false);
		
		if(toShow) parent.getShowButton().setEnabled(true);
		else parent.getShowButton().setEnabled(false);

	}

	public void assignTo(String source) {
		
		ListItem item;
		for(int i=0; i<((DefaultListModel)getModel()).size(); ++i){
			item = (ListItem) ((DefaultListModel)getModel()).elementAt(i);
			item.match(source);
		}
		
		repaint();
	}

	public void clear() {
		((DefaultListModel)getModel()).removeAllElements();
		dates.clear();
	}



}

