package inria.paperoles.ui;

import inria.paperoles.customization.CommandArgumentSet;
import inria.paperoles.customization.GestureManipulationCommand;
import inria.paperoles.gestures.recognition.GeneralizedGestureClass;
import inria.paperoles.ui.menus.JMarkingMenu;
import inria.paperoles.ui.menus.JMarkingMenuOption;

import java.awt.geom.Point2D;
import java.util.Vector;

import edu.umd.cs.piccolo.nodes.PPath;

public class FlatSubmenu extends PPath{

	InteractionSurface interactionSurface;
	JMarkingMenu menu;
	boolean hasOptions = false;
	
	public FlatSubmenu(InteractionSurface interactionSurface, Point2D absoluteCenter, 
			Point2D center, double radius, double angle, double step, 
			GesturePath gpath, CommandArgumentSet argumentSet, double scale){
		// needs to change
		
		hasOptions = !argumentSet.getArgument().isEmpty();
		if(!hasOptions) return;
		
		menu = new JMarkingMenu(absoluteCenter, 1);
		this.interactionSurface = interactionSurface;
		
		int i = 0;
		for(Object argument:argumentSet.getArgument()){	
			double dx = radius *Math.cos(angle - i*step);
			double dy = -radius*Math.sin(angle - i*step);
			
			JMarkingMenuOption visualLabel = new JMarkingMenuOption(argument.toString(),
					((GeneralizedGestureClass)argument).getDefinition().getIcon(), dx, dy);
			
			MarkingMenuOption option = new MarkingMenuOption(visualLabel, argumentSet.getCommand(), 
					argument, center.getX() + dx, center.getY() + dy);
			
			addChild(option);
			menu.addOption(visualLabel);
		
			i++;
		}
	}
	
	public boolean hasOptions(){
		return hasOptions;
	}
	
	public void activate(MarkingMenuOption option){
		option.addChild(this);
		interactionSurface.addMenu(menu);
	}
	
	public void disactivate(){
		removeFromParent();
		interactionSurface.remove(menu);
		interactionSurface.repaint();
	}

	public MarkingMenuOption findClosestOption(Point2D p) {
		double mindist2 = Double.MAX_VALUE;
		MarkingMenuOption closestOption = null;
		
		for(Object child:getChildrenReference()){
			MarkingMenuOption option = (MarkingMenuOption)child;
			
			double dist2 = Math.pow(option.getPosition().getX() - p.getX(), 2) + Math.pow(option.getPosition().getY() - p.getY(), 2);
			
			if(dist2 < mindist2){
				mindist2 = dist2;
				closestOption = option;
			}
		}
		
		return closestOption;
	}
	
}
