package inria.paperoles.ui;

import inria.paperoles.customization.ClassifyAllCommand;
import inria.paperoles.customization.ClassifyAllParametersCommand;
import inria.paperoles.customization.GestureDefinitionCommand;
import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.GestureReader;
import inria.paperoles.gestures.SingleInkStroke;
import inria.paperoles.gestures.StrokeAnalyzer;
import inria.paperoles.gestures.definition.SimpleGestureDefinition;
import inria.paperoles.gestures.recognition.GeneralizedGestureClass;
import inria.paperoles.gestures.recognition.ParameterRecognitionManager;
import inria.paperoles.gestures.recognition.RecognitionManager;
import inria.paperoles.grammar.GroupingGesture;
import inria.paperoles.grammar.SyntaxAnalyzer;
import inria.paperoles.io.ScoreDescriptionParser;
import inria.paperoles.io.Transformer;
import inria.paperoles.openmusic.OpenMusicClient;
import inria.paperoles.operations.Operation;
import inria.paperoles.scoremodel.MultiplePageSkeleton;
import inria.paperoles.scoremodel.ScoreSkeleton;
import inria.paperoles.tabletsupport.event.DrawingListener;
import inria.paperoles.uievents.ScrollListener;

import java.awt.BorderLayout;
import java.awt.CheckboxMenuItem;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.Dictionary;
import java.util.Random;
import java.util.Vector;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.plaf.basic.BasicToolBarUI;

import org.jpedal.PdfDecoder;

import papertoolkit.events.EventDispatcher;
import papertoolkit.events.EventHandler;
import papertoolkit.events.ExtendedPenEvent;
import papertoolkit.pen.ink.InkStroke;

import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;
import com.thoughtworks.xstream.XStream;
//import org.ximtec.igesture.algorithm.AlgorithmException;

import org.jpedal.PdfDecoder;
import org.jpedal.exception.PdfException;

public class GestureBrowser extends JFrame implements ActionListener{
	
//	final static String sheetFile = "sheets/score5.pdf";
	final static String sheetFile = "sheets/score-1.pdf";
	final static String skeletonFile = "descr/score-1.txt";
	
	final static String defaultDir = "./scores";
//	final static String skeletonFile = "score1.txt";
	
	//The commands supported by the toolbar
	private static final String TABLET = "tablet";
	private static final String ZOOM_IN = "zoomin";
	private static final String ZOOM_OUT = "zoomout";
	private static final String PENLAYER = "pen";
	private static final String COORDS = "coordinates";
	private static final String SKELETON = "skeleton";
	private static final String SCOPES = "scopes";
	private static final String REFRESH = "refresh";
	private static final String COMMIT = "om";
	private static final String OPEN_FILE = "fileopen";
	private static final String SAVE_PDF = "filesave";
	private static final String IMPORT = "import";
	private static final String EXPORT = "export";
	
	private static final String SAVE = "save";
	private static final String LOAD = "load";
	
	private static final String CLEAN = "clean";
	
	private static final String OUTPUT = "output";
	private static final String OM_PORT = "omport";
	private static final String RESET = "reset";
	private static final String STAFF_LENGTH = "stafflength";

	MultiplePageViewer viewer;
	GestureDictionaryBar dictionaryBar;
	RecognitionManager reconManager;
	ParameterRecognitionManager preconManager;
	FunctionsBar functionsBar;
//	AttributesBar attributesBar;
	
	GesturesContainer gcontainer;
	InteractionSurface interactionSurface;
	SyntaxAnalyzer syntaxAnalyzer;
	RubineController slider;
	
	PdfDecoder pdfDecoder;
	
	PaperTemplate currentPaperTemplate;
	JToolBar toolBar;
	
	DataListWindow dataFilesBar;
	
	XStream xstream = new XStream();
	OutputWindow outwindow;
	
	public GestureBrowser(int width, int height){
		super("Gesture Browser");
		
		pdfDecoder = new PdfDecoder();
		addMenuBar();
	
		reconManager = new RecognitionManager();
		preconManager = new ParameterRecognitionManager();

		gcontainer = new GesturesContainer(this);
		viewer = new MultiplePageViewer(this);
		
		ScoreContainer scoreContainer = new ScoreContainer(viewer, width, height);	
        interactionSurface = new InteractionSurface(scoreContainer, this);
 
        toolBar = new JToolBar("Tools");
        
        toolBar.setOrientation(JToolBar.VERTICAL);
        toolBar.setMargin(new Insets(1, 1, 1, 1));
        toolBar.setFloatable(false);
        
        addButtons(toolBar);
        
        add(toolBar, BorderLayout.LINE_START);
        add(interactionSurface, BorderLayout.CENTER);
        //add(scoreContainer, BorderLayout.CENTER);		    
   
        // TODO
		dictionaryBar = new GestureDictionaryBar(this);
		dictionaryBar.setFloatable(false);

//		attributesBar = new AttributesBar(this);
//		attributesBar.setFloatable(false);
		functionsBar = new FunctionsBar(this);
		functionsBar.setFloatable(false);
	
		JPanel toolbarsPanel = new JPanel();
		toolbarsPanel.setLayout(new BoxLayout(toolbarsPanel, BoxLayout.Y_AXIS));

		toolbarsPanel.add(functionsBar);
		toolbarsPanel.add(dictionaryBar);
//		toolbarsPanel.add(attributesBar);
		
		dictionaryBar.setAlignmentX(LEFT_ALIGNMENT);
//		attributesBar.setAlignmentX(LEFT_ALIGNMENT);
		functionsBar.setAlignmentX(LEFT_ALIGNMENT);
		
		add(toolbarsPanel, BorderLayout.LINE_END);
		
		this.setLocation(GlobalParameters.FRAME_OFFSET_X, GlobalParameters.FRAME_OFFSET_Y);

		dataFilesBar = new DataListWindow(this, GlobalParameters.FRAME_OFFSET_X + width, 
        		GlobalParameters.FRAME_OFFSET_Y + height);
    		
        
		outwindow = new OutputWindow(this, 500, 10);
		
		setBackground(Color.white);
		pack();
		setVisible(true);
		
//		importDefinitionsFrom(new File(GlobalParameters.DEFINITIONS_FILE), true);
		
		addComponentListener(new ComponentListener(){

			public void componentResized(ComponentEvent e) { 
				interactionSurface.updateSize();
			}

			public void componentHidden(ComponentEvent e) {
				// TODO Auto-generated method stub
			}

			public void componentMoved(ComponentEvent e) { 
				this.componentResized(e);
				
			}

			public void componentShown(ComponentEvent e) {
				// TODO Auto-generated method stub
				
			}
			
		});	
		
		//openScorePages(new File(skeletonFile), new File(sheetFile));
		
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
			//UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedLookAndFeelException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	GestureReader gestureReader;
	String sourceID = "_";
	public void openScorePages(File descrfile, File pdfFile){

			ScoreDescriptionParser reader = new ScoreDescriptionParser(descrfile);
		//	Transformer reader = new Transformer(skeletonFile);
			BufferedImage pageImage = null;
			
			MultiplePageSkeleton skeleton;
			try {
				skeleton = reader.build(currentPaperTemplate == null?GlobalParameters.OFFSET_X:currentPaperTemplate.getOffsetX(), 
						currentPaperTemplate == null?GlobalParameters.OFFSET_Y:currentPaperTemplate.getOffsetY());
				
				sourceID = skeleton.getSource();
			} 
			catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				
				return;
			} 
	
			viewer.clear();
//System.err.println(skeleton.elements());			
			StrokeAnalyzer strokeAnalyzer = new StrokeAnalyzer();
			syntaxAnalyzer = new SyntaxAnalyzer(skeleton);
			viewer.setSyntaxAnalyzer(syntaxAnalyzer);
			slider.setSyntaxAnalyzer(syntaxAnalyzer);
			viewer.setSkeleton(skeleton);
			
			viewer.setAnalyzers(strokeAnalyzer, syntaxAnalyzer);
			
			/////////	just for graph paper	
			if(sourceID.startsWith("millimetr")){
				ScoreViewer scoreViewer = new ScoreViewer(this, 1, pageImage, skeleton.elementAt(0), isVerticallyOriented(),
						currentPaperTemplate);
				
				scoreViewer.gesturesCanvas.setAnalysisHandlers(strokeAnalyzer, syntaxAnalyzer);
				viewer.addPageViewer(scoreViewer);
			}
			else {
				try{
					pdfDecoder.openPdfFile(pdfFile.getPath());
					
					for(int i = 0; i< skeleton.size(); ++i){
						ScoreSkeleton pageSkeleton = skeleton.elementAt(i);
						if(!pageSkeleton.isGraphPage()){
							if(!GlobalParameters.isTest) 
								pageImage = pdfDecoder.getPageAsImage(viewer.getScoreViewersNum() + 1);
							
							if(!isVerticallyOriented())
								pageImage = rotate90(pageImage, true);
							
							ScoreViewer scoreViewer = new ScoreViewer(this, viewer.getScoreViewersNum() + 1, pageImage, pageSkeleton, isVerticallyOriented(),
									currentPaperTemplate);
							
							scoreViewer.gesturesCanvas.setAnalysisHandlers(strokeAnalyzer, syntaxAnalyzer);
							viewer.addPageViewer(scoreViewer);	
						}
						else {
							ScoreViewer scoreViewer = new ScoreViewer(this, /*viewer.getGraphViewersNum() +*/ 1, pageImage, pageSkeleton, isVerticallyOriented(),
								currentPaperTemplate);
						
							scoreViewer.gesturesCanvas.setAnalysisHandlers(strokeAnalyzer, syntaxAnalyzer);
					//		viewer.addGraphViewer(scoreViewer, viewer.getGraphViewersNum() + 1);			
						}
					}
				} catch (PdfException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}			

				for(int i=0; i<viewer.getScoreViewersNum(); ++i){
					ScoreViewer scoreViewer = viewer.getPageViewers().elementAt(i);
					
	/*				for(ScoreViewer gviewer:viewer.getGraphViewers())
						scoreViewer.addGraphViewer(gviewer);*/
					
					scoreViewer.addGraphViewerSeparator();
				}
			}
			
	//////		
					        
			for(ScoreViewer sviewer:viewer.getPageViewers()){
				gcontainer.add(sviewer.gesturesCanvas);
				sviewer.gesturesCanvas.eventHandler.setInteractionSurface(interactionSurface);
			}	
	
	/*		for(ScoreViewer sviewer:viewer.getGraphViewers()){
				gcontainer.add(sviewer.gesturesCanvas);
			}
		*/	
			
			gestureReader = new GestureReader(viewer);
			
			activateButtons(true);
			interactionSurface.update();

			/*
			try {
				UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

			} catch (ClassNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InstantiationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (UnsupportedLookAndFeelException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			*/
			validate();
			
			saveAsPDF.setEnabled(true);
			resetItem.setEnabled(true);
			
			dataFilesBar.update();
	}

	
	public EventHandler getEventHandler(){
		return gestureReader.getEventHandler();
	}
	
	private BufferedImage rotate90(BufferedImage bi, boolean clockwise)
	{
		int width = bi.getWidth();
		int height = bi.getHeight();
		
		BufferedImage biFlip = new BufferedImage(height, width, bi.getType());
		
		for(int i=0; i<width; i++)
			for(int j=0; j<height; j++){
				if(clockwise) biFlip.setRGB(height-1-j, i, bi.getRGB(i, j));
				else biFlip.setRGB(j, width-1-i, bi.getRGB(i, j));
			}
		
		return biFlip;
	}
		
	public RecognitionManager getRecognitionManager(){
		return reconManager;
	}
	
	public ParameterRecognitionManager getParameterRecognitionManager(){
		return preconManager;
	}
	
	public OperationsList getOperationsList(){
		return functionsBar.getOperationList();
	}
	
	public GestureList getGestureList(){
		return dictionaryBar.getGestureList();
	}
	
/*	public GestureList getParameterList(){
		return attributesBar.getAttributesList();
	}
*/	
	//TODO
	public GesturesContainer getGesturesContainer(){			
		return gcontainer;
	}
	
	private void activateButtons(boolean activate){
		Component[] components = toolBar.getComponents();
		for(int i=0; i<components.length; ++i)
			components[i].setEnabled(activate);
	}
	
	public void refresh(){
		viewer.refresh();
		viewer.showScopes(scopesVisible);
	}

	
	protected void addButtons(JToolBar toolBar) {
		toolBar.setLayout(new BoxLayout(toolBar, BoxLayout.Y_AXIS));
		
	    JButton button = null;

	    /*
	    ///first button
	    button = createButton("refresh", REFRESH,
                "Refresh",
                "refresh");

	    button.setFont(new Font("Arial", Font.ITALIC, 10));
	    button.setSelected(false);
	    button.setAlignmentX(CENTER_ALIGNMENT);
	    toolBar.add(button); 
	    button.setEnabled(false);
	    
	    //second button
	    toolBar.addSeparator();*/
	    button = createButton("zoomin", ZOOM_IN,
	                                  "Zoom in the score",
	                                  "Zoom In");
	    toolBar.add(button);
	    button.setAlignmentX(CENTER_ALIGNMENT);
	    toolBar.addSeparator();
	    button.setEnabled(false);
	    
	    //...
	    button = createButton("zoomout", ZOOM_OUT,
	                                  "Zoom out the score",
	                                  "Zoom Out");
	    toolBar.add(button);
	    button.setEnabled(false);

	    button.setAlignmentX(CENTER_ALIGNMENT); 
	    
////	    
	    ///Add 
	    toolBar.addSeparator();
	    button = createButton("output", OUTPUT,
                "Show output window",
                "output");

	    button.setSelected(false);
	    toolBar.add(button);  	  
	    button.setAlignmentX(CENTER_ALIGNMENT);
	    button.setEnabled(true);	
	    
	    ///Add 
	    toolBar.addSeparator();
	    button = createButton("tablet", TABLET,
                "Draw with tablet/mouse",
                "tablet");

	    button.setSelected(false);
	    toolBar.add(button);  	  
	    button.setAlignmentX(CENTER_ALIGNMENT);
	    button.setEnabled(false);	
	    
	    
/*	    button.setSelected(false);
	    toolBar.add(button);  	  
	    button.setAlignmentX(CENTER_ALIGNMENT);
	    button.setEnabled(false);
*/	    
	    /*
	    ///Add 
	    toolBar.addSeparator();
	    button = createButton("skeleton", SKELETON,
                "Show score skeleton",
                "skeleton");

	    button.setSelected(false);
	    toolBar.add(button);  	  
	    button.setAlignmentX(CENTER_ALIGNMENT);
	    button.setEnabled(false);
	    
	    
	    toolBar.addSeparator();
	    button = createButton("coords", COORDS,
                "Show mouse coordinates",
                "(x, y)");

	    button.setSelected(false);
	    toolBar.add(button);  
	    button.setAlignmentX(CENTER_ALIGNMENT);	    
	    */
	    /*
	    ///Add 
	    toolBar.addSeparator();
	    button = createButton("scopes", SCOPES,
                "Show scopes",
                "scopes");       

	    button.setSelected(false);
	    toolBar.add(button);  	 
	    button.setAlignmentX(CENTER_ALIGNMENT);
	    button.setEnabled(false);
	    */
	    
	    ///Add 
	    toolBar.addSeparator();
	    button = createButton("om", COMMIT,
                "Send OSC to OM",
                "commit");

	    button.setFont(new Font("Arial", Font.ITALIC, 10));
	    button.setSelected(false);
	    toolBar.add(button); 
	    button.setAlignmentX(CENTER_ALIGNMENT);
	    button.setEnabled(false);
	    
	    /*
	    ///Erease gesture classes
	    toolBar.addSeparator();
	    button = createButton("refresh", REFRESH,
                "Clean gesture classes",
                "refresh");

	    button.setFont(new Font("Arial", Font.ITALIC, 10));
	    button.setSelected(false);
	    button.setAlignmentX(CENTER_ALIGNMENT);
	    toolBar.add(button); 
	    button.setEnabled(false);
	    
	    //second button
	    toolBar.addSeparator();
	    */
	    
	    ///slider....
	    toolBar.addSeparator();
	    slider = new RubineController(gcontainer, 
	    		GlobalParameters.DEFAULT_RECOGNIZER_DISTANCE);
	    toolBar.add(slider);
	    slider.setAlignmentX(CENTER_ALIGNMENT);
	    slider.setEnabled(false);
	    


	}

	protected JButton createButton(String iconName,
	                                       String actionCommand,
	                                       String toolTipText,
	                                       String altText) {
	    //Look for the image.
	    String imgLocation = null, simgLocation = null;
	    if(iconName!=null){
	    	imgLocation = GlobalParameters.ICONS_FOLDER + iconName + ".gif";
	    	simgLocation = GlobalParameters.ICONS_FOLDER + iconName + "-down.gif";
	    }

	    //Create and initialize the button.
	    JButton button = new JButton();
	    button.setActionCommand(actionCommand);
	    button.setToolTipText(toolTipText);
	    button.addActionListener(this);
	    button.setBorder(BorderFactory.createLineBorder(Color.gray, 1));
	    
	    if(iconName!=null){
	    	button.setIcon(new ImageIcon(imgLocation, altText));
	    	button.setPressedIcon(new ImageIcon(simgLocation, altText));
	    	button.setSelectedIcon(new ImageIcon(simgLocation, altText));
	    }
	    else button.setText(toolTipText);

	    return button;
	}
	
	JMenuItem exitItem, openItem, saveAsPDF, importItem, exportItem, resetItem;
	void addMenuBar(){
		JMenuBar menuBar = new JMenuBar();
		JMenu fileMenu = new JMenu("File");
		
		exitItem = new JMenuItem("Quit");
		exitItem.addActionListener( new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});	
		
		openItem = new JMenuItem("Open File...");
		openItem.addActionListener(this);
		openItem.setActionCommand(OPEN_FILE);
		
		saveAsPDF = new JMenuItem("Print To PDF...");
		saveAsPDF.addActionListener(this);
		saveAsPDF.setActionCommand(SAVE_PDF);
		
		importItem = new JMenuItem("Import Definitions...");
		importItem.addActionListener(this);
		importItem.setActionCommand(IMPORT);
		
		exportItem = new JMenuItem("Export Definitions...");
		exportItem.addActionListener(this);
		exportItem.setActionCommand(EXPORT);
	
		resetItem = new JMenuItem("Erase All");
		resetItem.addActionListener(this);
		resetItem.setActionCommand(RESET);
		
		fileMenu.add(openItem);
		fileMenu.add(resetItem);
		resetItem.setEnabled(false);
		
		fileMenu.add(saveAsPDF);
		saveAsPDF.setEnabled(false);
		
		fileMenu.addSeparator();
		fileMenu.add(importItem);
		fileMenu.add(exportItem);

		fileMenu.addSeparator();
		fileMenu.add(exitItem);		
		menuBar.add(fileMenu);
		
		JMenu format = new JMenu("Preferences"); 
		JMenuItem omPort = new JMenuItem("Configure OM Connection...");
		omPort.addActionListener(this);
		omPort.setActionCommand(OM_PORT);
		format.add(omPort);
		
		JMenuItem staffLength = new JMenuItem("Set Staff Duration...");
		staffLength.addActionListener(this);
		staffLength.setActionCommand(STAFF_LENGTH);
		format.add(staffLength);
		
		
		docTemplates = new JMenu("Score templates");
		try {
			addTemplates(docTemplates);
		} catch (IOException e1) {
		
			//e1.printStackTrace();
		}

		format.add(docTemplates);
		menuBar.add(format);
		
		this.setJMenuBar(menuBar);
	}

	JMenu docTemplates;
	PaperTemplates paperTemplates;
    private void addTemplates(JMenu docTemplates) throws IOException {
    	BufferedReader input = new BufferedReader(new FileReader(new File(GlobalParameters.DOC_FORMATS_FILE)));
    	
    	String line;
    	paperTemplates = new PaperTemplates();
    	
    	while((line = input.readLine()) !=null){
    		if(line.trim().startsWith("#") || line.trim().isEmpty()) continue;
    		
    		PaperTemplate ptemplate = paperTemplates.addTemplate(line);
    		JCheckBoxMenuItem item = new JCheckBoxMenuItem(ptemplate.id);
    		docTemplates.add(item);
    		
    		item.addActionListener(this);
    		item.setActionCommand(item.getName());
    		
    		if(ptemplate.isDefault())
    			item.setState(true);
    	}
    	
    	input.close();
    	
    	currentPaperTemplate = paperTemplates.getDefaultTemplate();
	}

    boolean scopesVisible = false;
	public void actionPerformed(ActionEvent e) {
        String cmd = e.getActionCommand();

        // Handle each button.
        if (ZOOM_IN.equals(cmd)) { //first button clicked
        	viewer.zoom(true);
        }
        else if (ZOOM_OUT.equals(cmd)) { // second button clicked
        	viewer.zoom(false);
        }
        /*
        else if(PENLAYER.equals(cmd)){
        	boolean selected = ((JButton)e.getSource()).isSelected();
        	
        	viewer.activatePenLayer(!selected);
        	((JButton)e.getSource()).setSelected(!selected);
        }
        else if(COORDS.equals(cmd)){
        	boolean selected = ((JButton)e.getSource()).isSelected();
        	
        	viewer.activateMouseCoords(!selected);
        	((JButton)e.getSource()).setSelected(!selected);
        }
        else if(SKELETON.equals(cmd)){
        	boolean selected = ((JButton)e.getSource()).isSelected();
        	
        	viewer.showSkeleton(!selected);
        	((JButton)e.getSource()).setSelected(!selected);
        }
        else if(SCOPES.equals(cmd)){
        	boolean selected = ((JButton)e.getSource()).isSelected();
        	
        	scopesVisible = !selected;
        	viewer.showScopes(!selected);
        	((JButton)e.getSource()).setSelected(!selected);
        }*/
        else if(REFRESH.equals(cmd)){
        	resetDefinitions();
        }
        else if(RESET.equals(cmd)){
        	eraseAll();
        }
        else if(COMMIT.equals(cmd)){
        	commit();
        }
        else if(TABLET.equals(cmd)){
        	boolean selected = ((JButton)e.getSource()).isSelected();
        	
        	enableTablet(!selected);
        	((JButton)e.getSource()).setSelected(!selected);
        }
        else if(OM_PORT.equals(cmd)){
        	boolean invalid = true;
        	while(invalid){
        		String port = JOptionPane.showInputDialog("Port of OpenMusic OSC Server", OpenMusicClient.PORT);
        		if(port == null) return;
        		try{
        			OpenMusicClient.PORT = Integer.parseInt(port);
        			invalid = false;
        		}
        		catch(NumberFormatException fe){}
        	}
        }
        else if(STAFF_LENGTH.equals(cmd)){
        	boolean invalid = true;
        	while(invalid){
        		String length = JOptionPane.showInputDialog("Length of a staff (in milliseconds)", GlobalParameters.STAFF_LENGTH);
        		if(length == null) return;
        		try{
        			GlobalParameters.STAFF_LENGTH = Integer.parseInt(length);
        			invalid = false;
        		}
        		catch(NumberFormatException fe){}
        	}
        }
        else if(OUTPUT.equals(cmd)){
        	if(!outwindow.isVisible()){
        		outwindow.setVisible(true);
        	}
        }
        else if (OPEN_FILE.equals(cmd)) { 
        	JFileChooser fc = new JFileChooser();
        	fc.setCurrentDirectory(new File(defaultDir));
        	
            fc.setFileFilter(new FileFilter(){
				public boolean accept(File file) {
					if(file.getName().toLowerCase().endsWith("pdf") 
							|| file.isDirectory()) return true;
					return false;
				}

				public String getDescription() {
					// TODO Auto-generated method stub
					return "pdf documents";
				}            	
            });
            
            int returnVal = fc.showOpenDialog(this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {   
            	final File pdffile = fc.getSelectedFile();
            	final File descrfile = new File(pdffile.getPath().substring(0, pdffile.getPath().lastIndexOf('.')) + ".txt");
            	
            	final BusyDialog dialog = new BusyDialog(this, "please wait...");
            	new Thread(){
            		public void run(){
            			openScorePages(descrfile, pdffile);
            			if(dialog!=null) dialog.setVisible(false);
            		}
            	}.start();
            	
            	dialog.setVisible(true);
            } else return;
        }
        else if(SAVE_PDF.equals(cmd)){
        	JFileChooser fc = new JFileChooser();
        	fc.setDialogTitle("Save As PDF");
        	
            fc.setFileFilter(new FileFilter(){
				public boolean accept(File file) {
					if(file.getName().toLowerCase().endsWith("pdf") || file.isDirectory()) return true;
					return false;
				}

				public String getDescription() {
					// TODO Auto-generated method stub
					return "pdf files";
				}            	
            });
            
            int returnVal = fc.showSaveDialog(this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
            	File file = fc.getSelectedFile();
            	if(!file.getName().toLowerCase().endsWith("pdf"))
            		file = new File(file.getPath().concat(".pdf"));
            	
            	final File pdfFile = file;
            		
            	final BusyDialog dialog = new BusyDialog(this, "saving...");
            	new Thread(){
            		public void run(){
                    	saveScoreTo(pdfFile);
            			if(dialog!=null) dialog.setVisible(false);
            		}
            	}.start();
            	
            	dialog.setVisible(true);
            } else return;
        }
        else if (EXPORT.equals(cmd)) { 
        	JFileChooser fc = new JFileChooser();
        	fc.setDialogTitle("Export Gesture Classes and Operations");
        	
            fc.setFileFilter(new FileFilter(){
				public boolean accept(File file) {
					if(file.getName().toLowerCase().endsWith("def") || file.isDirectory()) return true;
					return false;
				}

				public String getDescription() {
					// TODO Auto-generated method stub
					return "*.def files";
				}            	
            });
            
            int returnVal = fc.showSaveDialog(this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
            	File file = fc.getSelectedFile();
            	if(!file.getName().toLowerCase().endsWith("def"))
            		file = new File(file.getPath().concat(".def"));
            	
            	saveDefinitionsTo(file);
            } else return;
        }
        
        else if (IMPORT.equals(cmd)) { 
        	JFileChooser fc = new JFileChooser();
        	fc.setDialogTitle("Import Gesture Classes and Operations");
        	
            fc.setFileFilter(new FileFilter(){
				public boolean accept(File file) {
					if(file.getName().toLowerCase().endsWith("def") || file.isDirectory()) return true;
					return false;
				}

				public String getDescription() {
					// TODO Auto-generated method stub
					return "*.def files";
				}            	
            });
            
            int returnVal = fc.showOpenDialog(this);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
            	importDefinitionsFrom(fc.getSelectedFile(), false);
            } else return;
        }
        else if(paperTemplates.containsKey(cmd)){
        	currentPaperTemplate = paperTemplates.getTemplate(cmd);
        	
        	for(int i=0; i < docTemplates.getMenuComponentCount(); ++i){
        		JCheckBoxMenuItem menuItem = (JCheckBoxMenuItem)docTemplates.getMenuComponent(i);
        		
        		if(menuItem.getText().equals(cmd)) menuItem.setState(true); 
        		else menuItem.setState(false);
        	
        	}
        }
        
    }
	
	private void eraseAll() {
		//viewer.clear();
		viewer.cleanWrittenStrokes();
	}


	private void saveScoreTo(File file) {
    	ScoreViewer pageViewer = viewer.getPageViewers().elementAt(0);
    	float width, height;
    
    	if(isVerticallyOriented()){
    		width = pageViewer.getWidth();
    		height = pageViewer.getHeight();
    	}	
    	else{
    		width = pageViewer.getHeight();
    		height = pageViewer.getWidth();
    	}
    	
    	Document document = new Document(new com.lowagie.text.Rectangle(width, height));
    	try { 
    		  PdfWriter writer = PdfWriter.getInstance(document , new FileOutputStream(file));
    		  // we add some meta information to the document
    		  document.addSubject(""); 
    		  document.open(); 

    		  for(ScoreViewer pviewer:viewer.getPageViewers()){
    		  
    			  PdfContentByte cb = writer.getDirectContent();
    			  PdfTemplate tp = cb.createTemplate(width, height);
    	      	
    			  //if(pviewer!=pageViewer) 
    			  BufferedImage image = (BufferedImage )pviewer.getRoot().toImage();
  			  
    			  Graphics2D g2 = tp.createGraphics(width, height);
    			  
  				  if(!isVerticallyOriented())
						image = rotate90(image, false);
  					
    			  g2.drawImage(image, 0, 0, null);

    			  g2.dispose();
    			  cb.addTemplate(tp, 0, 0);
    			
    			  document.newPage();
    		  }
    		  
    	  } catch(DocumentException e) {
    		  System.err.println(e.getMessage());
    	  }
    	  catch(IOException e) {
    		  System.err.println(e.getMessage());
    	  }
    	  document.close();
    }
    
    private boolean isVerticallyOriented() {
		if(currentPaperTemplate != null) return currentPaperTemplate.isPortrait;
    	return true;
	}

    
    Vector<GeneralizedGestureClass> defaultParameterClasses = null;
    
    
    private void resetDefinitions() {
    	getGestureList().loadGestures(null);
    	reconManager = new RecognitionManager();
    	preconManager = new ParameterRecognitionManager();
    
    	viewer.resetDefinitions();    	
    }
    
    public void removeGestureClass(GeneralizedGestureClass gestureClass){
    	if(gestureClass.getDefinition().isCircled())
    		preconManager.removeGestureClass(gestureClass);
    	else reconManager.removeGestureClass(gestureClass);
    	
    	viewer.resetDefinitions();
    }
    
	private void importDefinitionsFrom(File file, boolean isDefault) {

    	ObjectInputStream objectIn;
		try {
			objectIn = new ObjectInputStream(new BufferedInputStream(
					new FileInputStream(file)));
			
	    	getOperationsList().loadOperations((Vector<Operation>) xstream.fromXML((String)objectIn.readObject()));
			
			Vector<GeneralizedGestureClass> classes = (Vector<GeneralizedGestureClass>) xstream.fromXML((String)objectIn.readObject());
			
			getGestureList().loadGestures(classes);
			
			preconManager.loadClasses(classes);		
			reconManager.loadClasses(classes);
	
			/*
			classes = (Vector<GeneralizedGestureClass>) xstream.fromXML((String)objectIn.readObject());
			
			if(isDefault) defaultParameterClasses = classes;
			else if(defaultParameterClasses!=null) classes.addAll(defaultParameterClasses);
			*/
//			getParameterList().loadGestures(classes);
//			preconManager.loadClasses(classes);
			
	    	objectIn.close();
	    	
	    	viewer.resetDefinitions();
	    	
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
    private void saveDefinitionsTo(File file) {
    	ObjectOutputStream objectOut;
		try {
			objectOut = new ObjectOutputStream(new BufferedOutputStream(
					new FileOutputStream(file)));
	
	    	objectOut.writeObject(xstream.toXML(getOperationsList().getOperationsVector()));
			
	    	objectOut.writeObject(xstream.toXML(getGestureList().getGestures()));
//	    	objectOut.writeObject(xstream.toXML(getParameterList().getGestures()));
	    	objectOut.close(); 
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
    
    
    private void saveStrokesTo(File file) {
    	ObjectOutputStream objectOut;
		try {
			objectOut = new ObjectOutputStream(new BufferedOutputStream(
					new FileOutputStream(file)));
			
			Vector<Vector<InkStroke>> strokes = new Vector<Vector<InkStroke>>();
			for(ScoreViewer sviewer:viewer.scoreViewers)
				strokes.add(sviewer.getAcceptedStrokes());
			
	    	objectOut.writeObject(xstream.toXML(strokes));
	    	objectOut.close(); 
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
    
    public void cleanStrokes(){
    	viewer.cleanWrittenStrokes();
    }
    

  
    public String getSource(){
    	if(sourceID.equals("_")) return null;
    	else return sourceID;
    }
    
	private boolean commit(){
    	OpenMusicClient OMClient = null;
    	
    	try{
    		OMClient = new OpenMusicClient(getOperationsList().getOperationsMap());
    		
    		OMClient.submitRecognizedOperations(outwindow, viewer.getMatchedTokens());
    	}
    	catch(IOException e){	
    		e.printStackTrace();
    		JOptionPane.showMessageDialog(this, "Unable to open connection with Open Music");
    		return false;
    	}
		
		OMClient.kill();
    	return true;
    }
    
    public SyntaxAnalyzer getSyntaxAnalyzer(){
    	return viewer.analyzer;
    }
    
    public void updateGestureClass(GeneralizedGestureClass gclass, GesturePath gesture){
    	reconManager.addInstanceToClass(gesture.token.getFirstStroke(), gclass);
    	
   System.err.println("added gesture" ); 	
    }
    
    public void updateParameterClass(GeneralizedGestureClass gclass, GesturePath gesture){
    	preconManager.addInstanceToClass(((GroupingGesture)gesture.token).getGroup().combineStroke(), gclass);
    System.err.println("added parameter" ); 	
    }
    
	public GeneralizedGestureClass addGestureClass(GesturePath gesture,
			SimpleGestureDefinition definition) {
	
		GeneralizedGestureClass gclass = reconManager.createGestureClass(gesture.token.getFirstStroke(), definition);
		
		if(gclass == null){
			JOptionPane.showMessageDialog(this, 
					"Cannot create class. The symbol may be too tiny.");
			
			return null;
		}
		
		if(definition.isParameter());// getParameterList().addGesture(gclass); 
		else getGestureList().addGesture(gclass); 
		
		gesture.assignToGestureClass(gclass, 0, true);
		
		return gclass;		
	}
	
	public GeneralizedGestureClass addParameterClass(GesturePath gesture,
			SimpleGestureDefinition definition) {
		
		GeneralizedGestureClass gclass = preconManager.createGestureClass(((GroupingGesture)gesture.token).getGroup().combineStroke(), definition);
		
		if(gclass == null){
			JOptionPane.showMessageDialog(this, 
					"Cannot create class. The symbol may be too tiny.");
			
			return null;
		}
		
		if(definition.isParameter());// getParameterList().addGesture(gclass); 
		else getGestureList().addGesture(gclass); 
		
		gesture.assignToGestureClass(gclass, 0, true);
		
		return gclass;
	}
	
	DrawingListener drawingListener = null;
	private void enableTablet(boolean selected){
		Cursor normalCursor;
		
		if(!selected) normalCursor = new Cursor(Cursor.DEFAULT_CURSOR);
		else normalCursor = new Cursor(Cursor.CROSSHAIR_CURSOR);
		
		setCursor(normalCursor);
		
		for(ScoreViewer sviewer:viewer.getPageViewers()){
			if(selected){
				drawingListener = new DrawingListener(dataFilesBar, isVerticallyOriented());
				sviewer.addInputEventListener(drawingListener);
			}
			else if(drawingListener!=null){
				drawingListener.end();
				sviewer.removeInputEventListener(drawingListener);
				sviewer.cleanDrawingSurface();
				drawingListener = null;
			}
		}
	}
	
}
