package inria.paperoles.ui;

import inria.paperoles.customization.GestureFilteringCommand;
import inria.paperoles.gestures.definition.GestureDefinition;
import inria.paperoles.gestures.recognition.GeneralizedGestureClass;
import inria.paperoles.operations.Operation;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.ListCellRenderer;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.ximtec.igesture.core.GestureClass;

import sun.tools.jar.JarImageSource;

public class GestureList extends JList implements ListSelectionListener{

	GestureFilteringCommand filteringCommand;
	
	public GestureList(final GesturesContainer gcontainer){
		super(new DefaultListModel());
		
	//	setFixedCellWidth(2*GlobalParameters.DICTIONARY_ITEM_WIDTH + 10);
		setCellRenderer(new GestureCellRenderer());
		setVisibleRowCount(-1);
		setLayoutOrientation(JList.HORIZONTAL_WRAP);
		
		filteringCommand = new GestureFilteringCommand(gcontainer);
		addListSelectionListener(this);
		
		addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e) {
				if(e.getClickCount() < 2) return;
				
				editSelectedGesture(gcontainer.getBrowserFrame(), 
						e.getXOnScreen(), e.getYOnScreen());
			}
		});
		
		addKeyListener(new KeyAdapter(){
			
			boolean fpressed = false;
			public void keyPressed(KeyEvent e) {
				if(e.getKeyCode() == KeyEvent.VK_BACK_SPACE){
					ListItem selection = (ListItem)getSelectedValue();
					
					if(selection!=null){
						gcontainer.removeGestureClass(selection.getGestureClass());
						((DefaultListModel)getModel()).removeElement(selection);
					}
				}
				else if (e.getKeyCode() == KeyEvent.VK_F){
					Vector<GeneralizedGestureClass> selectedClasses = new Vector<GeneralizedGestureClass>();
					
					for(Object object:getSelectedValues()){
						GeneralizedGestureClass gclass = ((ListItem)object).getGestureClass();
					
						selectedClasses.add(gclass);
					}
					
					if(!selectedClasses.isEmpty()){
						filteringCommand.activate(selectedClasses);
						fpressed = true;
					}
				}
			}		
			
			public void keyReleased(KeyEvent e) {
				if (e.getKeyCode() == KeyEvent.VK_F && fpressed){
					fpressed = false;
					filteringCommand.activate(null);
				}
			}
		});
	}
	
	private void editSelectedGesture(GestureBrowser browser, int x, int y){
		ListItem selection = (ListItem)getSelectedValue();
		if(selection == null) return;
			
		(new GestureDefinitionDialog(browser, 
				selection.getGestureClass(), null, x - 200, y - 200)).setVisible(true);
		
		selection.update();
	}
	
	
	public void updateSelectedGesture(){
		ListItem selection = (ListItem)getSelectedValue();
		if(selection == null) return;
	
		selection.update();
	}
	
	public void addGesture(GeneralizedGestureClass gclass){
		ListItem listItem = new ListItem(gclass);//gclass.getName(), new ImageIcon(gclass.getDefinition().getImage()));
		
		int i = 0;
		for(; i< ((DefaultListModel)getModel()).getSize(); ++i){
			ListItem item = (ListItem)((DefaultListModel)getModel()).getElementAt(i);
			if(listItem.getLabel().compareTo(item.getLabel()) < 0) break;
		}
		
		((DefaultListModel)getModel()).insertElementAt(listItem, i);
	}
	
	public Vector<GeneralizedGestureClass> getGestures(){
		Vector<GeneralizedGestureClass> classes = new Vector<GeneralizedGestureClass>();
			
		for(int i =0; i< ((DefaultListModel)getModel()).getSize(); ++i){
			ListItem item = (ListItem)((DefaultListModel)getModel()).getElementAt(i);
			classes.add(item.getGestureClass());
		}
		
		return classes;
	}
	
	final static Color selectColor = GlobalParameters.LIST_SELECTION_COLOR;
		
	private class GestureCellRenderer extends JLabel implements ListCellRenderer{
		public GestureCellRenderer(){
			super();
			setFont(new Font("Arial", Font.PLAIN, 10));
			setForeground(Color.black);
			setOpaque(true);
	       	setVerticalTextPosition(JLabel.BOTTOM);
        	setHorizontalTextPosition(JLabel.CENTER);
    		setBorder(BorderFactory.createEmptyBorder(6,6,6,6));
    		setPreferredSize(new Dimension(GlobalParameters.DICTIONARY_ITEM_WIDTH + 15/*+ 23*/, 
    				GlobalParameters.DICTIONARY_ITEM_HEIGHT + 20));
		}

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
        {
        	
        	ListItem item = (ListItem)value;
        	
        	if(item.hasOperation())
        		setText(":"+item.getOperation());
        	else setText(item.getLabel());
            setIcon(item.gimage);

            setBackground(isSelected ?  selectColor: Color.white);
           
            //setForeground(isSelected ? Color.white : Color.black);
            return this;
        }
	}
	
    private class ListItem
    {
    	
    	GeneralizedGestureClass gclass;
    	
    	Icon gimage;
    	String label, operation = null;
        
        ListItem(GeneralizedGestureClass gclass)
        {
            this.gclass = gclass;
            gimage = gclass.getDefinition().getImage();
    
            label = gclass.getName();
            
            if(gclass.getDefinition().getOperation() != null)
            	operation = gclass.getDefinition().getOperation(); 
        }
  
        public void update(){ 
        	label = gclass.getName();
        	
            if(gclass.getDefinition().getOperation() != null)
            	operation = gclass.getDefinition().getOperation(); 
            else operation = null;
        }
        
        public GeneralizedGestureClass getGestureClass(){
        	return gclass;
        }
 
        public String getLabel(){
        	return label;
        }
        
        public String getOperation(){
        	return operation;
        }
        
        public boolean hasOperation(){
        	return operation!=null && !operation.isEmpty();
        }
        
        public Icon getImage(){
        	return gimage;
        }
    }


    private GeneralizedGestureClass getSelectedClass(){
    	Object[] selection = getSelectedValues();
    	if(selection != null && selection.length > 0) return ((ListItem)selection[0]).getGestureClass();
    	else return null;
    }
    
	public void valueChanged(ListSelectionEvent e) {/*
		if (e.getValueIsAdjusting() == false) {
			Vector<GeneralizedGestureClass> selectedClasses = new Vector<GeneralizedGestureClass>();
			
			for(Object object:getSelectedValues()){
				GeneralizedGestureClass gclass = ((ListItem)object).getGestureClass();
			
				selectedClasses.add(gclass);
			}
			
			filteringCommand.activate(selectedClasses);
		}*/
	}

	public void loadGestures(Vector<GeneralizedGestureClass> classes) {
		((DefaultListModel)getModel()).clear();
				
		if(classes!=null)
			for(GeneralizedGestureClass gclass:classes)
				addGesture(gclass);
	}
}
