package inria.paperoles.ui;

import inria.paperoles.gestures.GeneralInkStroke;
import inria.paperoles.gestures.SingleStrokeAnalysisResult;
import inria.paperoles.gestures.definition.SimpleGestureDefinition;
import inria.paperoles.gestures.recognition.GeneralizedGestureClass;
import inria.paperoles.grammar.AssociatedComplexGesture;
import inria.paperoles.grammar.BlockToken;
import inria.paperoles.grammar.ChordsScope;
import inria.paperoles.grammar.ComplexGesture;
import inria.paperoles.grammar.ConnectedComplexGesture;
import inria.paperoles.grammar.GroupingGesture;
import inria.paperoles.grammar.Scope;
import inria.paperoles.grammar.ScopingPair;
import inria.paperoles.grammar.Token;
import inria.paperoles.operations.MatchedOperation;
import inria.paperoles.operations.Operation;
import inria.paperoles.operations.OperationArgument;
import inria.paperoles.uievents.GestureEventHandler;

import java.awt.Color;
import java.awt.Font;
import java.awt.Image;
import java.awt.Paint;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;
import java.util.Collection;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFrame;

import org.ximtec.igesture.core.Result;
import org.ximtec.igesture.core.ResultSet;

import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.nodes.PText;
import edu.umd.cs.piccolo.util.PPickPath;

public class GesturePath extends PPath {


	final static Color DEFAULT_PAINT = new Color(180, 240, 100, 60);

	final static Color STROKE_RECOGNIZED = new Color(90, 90, 100);
	final static Color STROKE_RECOGNIZED_PARAMETER = new Color(50, 130, 80);
	
	final static Color PAINT_RECOGNIZED = new Color(140, 140, 200, 60);
	final static Color PAINT_RECOGNIZED_PARAMETER = new Color(140, 200, 200, 60);

	final static Color STROKE_DISABLED = new Color(220, 220, 220);
	final static Color PAINT_DISABLED = null;
	
	final static Color TOOLTIP_COLOR = Color.blue;
	
	private static final Color SCOPE_STROKE_COLOR = Color.blue;
	
	Color inkStrokeColor = GeneralInkStroke.STROKE_COLOR;
	
	// I will probably have to remove this property as i don' t really need it.
//	SingleStrokeAnalysisResult strokeDescription;

	PText label;
	
	PPath body;
	PPath scope;	
	Color paint = DEFAULT_PAINT;
	Color stroke_paint;
	
	Token token;
	
	boolean disabled = false;
	
	GeneralizedGestureClass gestureClass;
	GeneralizedGestureClass toApplyClass;
	boolean classified = false;
	
	GesturesCanvas gcanvas;
	
	double reconDistance = 0;
	
//	AddToSamplesButton samplesButton = null;
	
	public GesturePath(){
		super();
	}

	public GesturePath(GesturesCanvas gcanvas, Token token){ 
		this.token = token;
		body = new PPath();
		this.gcanvas = gcanvas;
		
		if(token.getFirstStroke() !=null)
			inkStrokeColor = token.getFirstStroke().getColor();
		
		stroke_paint = inkStrokeColor;
		
		if(!token.isLeaf()){
			Collection<Token> tokens = token.getSubTokens();
			
			for(Token t:tokens){
				GesturePath gpath = new GesturePath(gcanvas, t); 
				t.addGesture(gcanvas, gpath);
				body.addChild(gpath);
			}
		}
		else{
			if(token.getCanvas() == gcanvas)
				for(Shape shape: token.getFirstStroke().getShapes()){ 
					PPath path = new PPath(shape);  
					path.setStrokePaint(stroke_paint);

					body.addChild(path);		
				}
			
			body.addChild(new GestureHandle(token.getFirstStroke()));
					
		}

		Scope scopeArea = token.getScope(); 
		
		if(scopeArea != null && scopeArea.inGesturesCanvas(gcanvas) && !scopeArea.getShape(gcanvas).getBounds().isEmpty()){
			scope = new PPath(scopeArea.getShape(gcanvas));	
			scope.setPaint(paint);
			if(scopeArea instanceof ChordsScope) scope.setStrokePaint(Color.blue);	
			else scope.setStrokePaint(null);
			
			addChild(scope);
			
			scope.setPickable(false);
		}
		
		addChild(body);		
	}	
	
	public Token getToken(){
		return token;
	}
	
	
	public GesturesCanvas getCanvas(){
		return gcanvas;
	}
	
	public void setFocus(){
	/*	if(token instanceof ComplexGesture && !(token instanceof GroupingGesture)){
			for(Token subtoken :((ComplexGesture)token).getSubTokens())
				subtoken.getGesturePath(gcanvas).setFocus();
		}*/
		
		setFocused(false);
		
		Collection<Token> linked = token.getLinkedTokens();
		if(linked == null) return;
		
		for(Token token:linked)
			if(token.getGesturePath(gcanvas) != null) token.getGesturePath(gcanvas).setFocused(true);
	}
	
	public void setFocused(boolean lightFocus){ 
		if(disabled || hasFocus) return;
		if(!lightFocus) hasFocus = true; 
		fixColor(lightFocus?getHalfDarker(paint):getDarker(paint), lightFocus?getHalfDarker(stroke_paint):getDarker(stroke_paint),
				SCOPE_STROKE_COLOR);
	
		for(GesturePath gpath:token.getOtherPaths(this)){
			gpath.fixColor(lightFocus?getHalfDarker(paint):getDarker(paint), lightFocus?getHalfDarker(stroke_paint):getDarker(stroke_paint),
					SCOPE_STROKE_COLOR);
		}
		
	}
	
	
	private Color getHalfDarker(Color c){
		if(c == null) return null;
		return new Color((int)(.9*c.getRed()), (int)(.7*c.getGreen()), (int)(.7*c.getBlue()), 
				Math.min(255, (int)(1*c.getAlpha())));
	}
	
	private Color getDarker(Color c){
		if(c == null) return null;
		return new Color((int)(.9*c.getRed()), (int)(.5*c.getGreen()), (int)(.5*c.getBlue()), 
				Math.min(255, (int)(1.2*c.getAlpha())));
	}
	
	
	public void setNormal(){ 
		/*if(token instanceof ComplexGesture && !(token instanceof GroupingGesture)){
			for(Token subtoken :((ComplexGesture)token).getSubTokens())
				subtoken.getGesturePath(gcanvas).setNormal();
		}
		else*/
//		if(samplesButton != null) samplesButton.setVisible(true);
		unfocus();
		
		Collection<Token> linked = token.getLinkedTokens();
		if(linked == null) return;
		
		for(Token token:linked)
			if(token.getGesturePath(gcanvas) != null) token.getGesturePath(gcanvas).unfocus();
	}
	
	public void unfocus(){ 
	//	if(token.getParent() != null) return;
		
		hasFocus = false;
	
		fixColor(paint, stroke_paint, SCOPE_STROKE_COLOR);
		
		for(GesturePath gpath:token.getOtherPaths(this)){
			gpath.fixColor(paint, stroke_paint, SCOPE_STROKE_COLOR);
		}
		
		disabled = false;
	}
	
	
	PPath tokenLabel = null;
	public void revealToken(){
		tokenLabel = createLabel(token.toString(),
				(int)body.getFullBounds().getCenterX(), 
				(int)body.getFullBounds().getY() - 10, Color.blue);
		addChild(tokenLabel);
	}
	
	private void hideToken(){	
		if(tokenLabel == null) return;
		
		removeChild(tokenLabel);
		tokenLabel = null;
	}
	
	
	PPath operationLabel = null;
	Color labelsColor = Color.black;
	public void revealOperation(){
		this.moveToFront();
		/*
		if(gestureClass.getDefinition().isParameter()){ // change that... highlight associated gestures
			System.err.println("Associated parameter: " + gestureClass.getDefinition().getName());
			return;
		}*/
		/*
		Operation operation = gestureClass.getDefinition().getOperation();

		if(operation!=null){
			Hashtable<OperationArgument, Collection<Token>> matchings = token.getMatchedOperation().getMatchings();
			
			/////////////////
			operationLabel = createLabel(gestureClass.getDefinition().getName() + ": " + operation.getName(),
					(int)body.getFullBounds().getCenterX(), 
					(int)body.getFullBounds().getY() - 10, TOOLTIP_COLOR);
			addChild(operationLabel);
			//////////////////
			
			ScoreViewer viewer = gcanvas.getViewer();
			for(OperationArgument arg: operation.getArguments()){ 
				Collection<Token> tokens = matchings.get(arg);
				if(tokens != null)
					for(Token token:tokens) {
						if(token != null)
							for(GesturePath path:token.getAllPaths()){
								path.revealOperationComponent(arg, this);
								
								if(path.gcanvas == gcanvas) continue;
								
								path.gcanvas.getViewer().highlightButton(viewer, true);
							}
					}
			}
			
		} 
		else {
			operationLabel = createLabel((gestureClass.getDefinition().isParameter()?"Parameter: ":"") + gestureClass.getDefinition().getName(),
					(int)body.getFullBounds().getCenterX(), 
					(int)body.getFullBounds().getY() - 10, TOOLTIP_COLOR);
			addChild(operationLabel);
		}*/
		operationLabel = createLabel(gestureClass.getDefinition().getName(), (int)body.getFullBounds().getCenterX(), (int)body.getFullBounds().getY() - 10, TOOLTIP_COLOR);
		addChild(operationLabel);
	}
	

	private PPath createLabel(String string, int x, int y, Color color){
		PPath box = new PPath();
		
		PText text = new PText(string);
		text.setFont(new Font("Arial", Font.PLAIN, 11));
		text.setTextPaint(color);
		
		box.setPaint(Color.white);
		box.setStrokePaint(color);
		box.addChild(text);
		
		box.setPathToRectangle((int)(x - text.getWidth()/2 - 2),
				(int)(y - text.getHeight()/2 - 2), 
				(int)text.getWidth() + 4, (int)text.getHeight() + 4);
		
		text.setX(box.getBounds().getX() + 2);
		text.setY(box.getBounds().getY() + 2);
		
		return box;
	}
	
	Vector<PPath> argumentLabels = new Vector<PPath>();
	void revealOperationComponent(OperationArgument arg, GesturePath gpath){
		if(gpath!=this) setFocused(true);
		
		argumentLabels.add(createLabel("$" + (arg.getIndex()+1), 
				(int)body.getFullBounds().getCenterX() + argumentLabels.size()*15, 
				(int)body.getFullBounds().getCenterY(), TOOLTIP_COLOR));
		
		addChild(argumentLabels.lastElement());
	}
	
	public void hideOperation(){	
		if(tokenLabel !=null){
			hideToken();
			return;
		}
		
		if(operationLabel == null) return;
		
		removeChild(operationLabel);
		operationLabel = null;	
	}
	
	void hideOperationComponent(GesturePath gpath){ 
		if(gpath!=this) setNormal();

		for(PPath argumentLabel:argumentLabels){
			removeChild(argumentLabel);
		}
		
		argumentLabels.removeAllElements();
	}
	
	
	private void fixColor(Color paint, Color stroke_paint, Color borders_paint){
	
		if(!token.isLeaf()){
			for(int i=0; i<body.getChildrenCount(); ++i){ 
				GesturePath path = (GesturePath)body.getChild(i);
				path.fixColor(paint, stroke_paint, borders_paint); 
			}
		}
		else{
			for(int i=0; i<body.getChildrenCount(); ++i){ 
				PPath path = (PPath)body.getChild(i);
				if(!(path instanceof GestureHandle)) path.setStrokePaint(stroke_paint);
			}
		}
		if(scope != null){
			scope.setPaint(paint);
			if(token.getScope() instanceof ChordsScope) scope.setStrokePaint(borders_paint);
		}
	}
	
	
	public void setScopeTo(Scope newscope){
		if(newscope == null) return;
		scope.setPathTo(newscope.getShape(gcanvas));
		if(newscope instanceof ChordsScope) scope.setStrokePaint(SCOPE_STROKE_COLOR);
		else scope.setStrokePaint(null);
	}
	
	
	public PPath getScope(){
		return scope;
	}
	

	public void setLabel(){

	}
	

	public boolean intersects(Rectangle2D localBounds) {
		if (localBounds == null) return true;
		return getBoundsReference().intersects(localBounds);
	}
	
	public void updateScope(){
	//	scope.setPathTo(scopeRepresentation.getScopeArea());
	}
	
		
	boolean hasFocus = false;
	
	public boolean hasFocus(){
		return hasFocus;
	}
	

	public void prepareScopeToChange(boolean toChange) {}


	public void addInputEventListener(GestureEventHandler eventHandler) {
		
		super.addInputEventListener(eventHandler);
	}

	
	public ImageIcon getTumbnail(int max_w, int max_h, boolean parameter){

		PPath path = new PPath();
		AffineTransform transf = new AffineTransform();
		double scale1, scale2;
		
		if(token instanceof BlockToken){
			Shape s1 = ((BlockToken)token).getLeft().getFirstStroke().getShape();
			scale1 = max_w/(s1.getBounds().getWidth()*3);
			scale2 = max_h/s1.getBounds().getHeight();
			scale1 = Math.min(1, Math.min(scale1, scale2));	
			
			transf.scale(scale1, scale1);
			s1 = transf.createTransformedShape(s1);
			
			transf = new AffineTransform();
			transf.rotate(Math.PI);
			//transf.translate(20, 0);
			Shape s2 = transf.createTransformedShape(s1);
			
			PPath p = new PPath(s1);
			p.setBounds(0, 0, s1.getBounds().getWidth(), s1.getBounds().getHeight());
			path.addChild(p);
			p.setStrokePaint(STROKE_RECOGNIZED);
			
			p = new PPath(s2);
			p.setBounds(2*s1.getBounds().getWidth(), 0, s2.getBounds().getWidth(), s2.getBounds().getHeight());
			p.setStrokePaint(STROKE_RECOGNIZED);
			path.addChild(p);
			
			return new ImageIcon(path.toImage());
		}
		
		GeneralInkStroke stroke;
		if(token instanceof GroupingGesture)
			stroke = ((GroupingGesture)token).getMixedStroke();
		else stroke = token.getFirstStroke();
	
		Shape shape = stroke.getShape();
		scale1 = max_w/shape.getBounds().getWidth();
		scale2 = max_h/shape.getBounds().getHeight();
		
		scale1 = Math.min(1.1, Math.min(scale1, scale2));	
	
		transf.scale(scale1, scale1);
		
		for(Shape s: stroke.getShapes()){ 
			s = transf.createTransformedShape(s);
			PPath p = new PPath(s);
			if(parameter) p.setStrokePaint(STROKE_RECOGNIZED_PARAMETER);
			else p.setStrokePaint(STROKE_RECOGNIZED);
			path.addChild(p);
		}
		
		return new ImageIcon(path.toImage());
	}
	
	// Needs to be deleted (1) from the visual space, and (2) from the symbol table 
	public void delete() { 

		if(token.getFirstStroke() != null){			
			((ScoreViewer)((PCamera)getRoot().getChild(0)).getComponent()).removeStroke(token.getFirstStroke());
		}
		
		removeFromParent();	
		token.removeFromSymbols(); 
		token.removeDependantGestures();
		gcanvas.getSyntaxAnalyzer().removeToken(token);
	
		for(GesturePath gpath:token.getOtherPaths(this)){
			gpath.removeFromParent();
		}
	}
	
	public void applyNewThresDistance(){
		if(toApplyClass == null) return;

		if(classified && reconDistance > gcanvas.reconDistanceThreshold
				|| !classified && reconDistance <= gcanvas.reconDistanceThreshold){
		
			classified = !classified;	
			refreshClass();
		}
	}
	
	
	public boolean updateClass(){ 
		if(gestureClass != null && classified 
				|| gestureClass == null && !classified) return false;
		
		assignClass(false);
		return true;
	}
	
	private void refreshClass(){ 
		if(classified){			
			if(toApplyClass.getDefinition().isParameter()){
				stroke_paint = STROKE_RECOGNIZED_PARAMETER;
				paint = PAINT_RECOGNIZED_PARAMETER;
			}
			else{
				stroke_paint = STROKE_RECOGNIZED;
				paint = PAINT_RECOGNIZED;
			}
				
			for(GesturePath gpath:token.getOtherPaths(this)){
				gpath.paint = PAINT_RECOGNIZED;
				gpath.stroke_paint = STROKE_RECOGNIZED;
			}
			
		}
		else {			
			paint = DEFAULT_PAINT;
			stroke_paint = inkStrokeColor ;
				
			for(GesturePath gpath:token.getOtherPaths(this)){
				gpath.paint = DEFAULT_PAINT;
				gpath.stroke_paint = inkStrokeColor ;
			}
		}
				
		setNormal();
	}
	
	public void assignToGestureClass(GeneralizedGestureClass gestureClass, double dist, boolean newClass) {
	
		toApplyClass = gestureClass;
		reconDistance = dist;
		
		if(gestureClass != null && !classList.contains(gestureClass)) classList.add(gestureClass);
		if(dist > gcanvas.reconDistanceThreshold) gestureClass = null;
			
		if(gestureClass!=null) classified = true;
		else classified = false;
			
		assignClass(newClass);
		
		refreshClass();
	}
	
	private void assignClass(boolean newClass){
		
//		boolean toAddButton = (!newClass  && toApplyClass !=null && toApplyClass != gestureClass);
		
		if(classified) gestureClass = toApplyClass;
		else gestureClass = null;
			
		for(GesturePath gpath:token.getOtherPaths(this))
			gpath.gestureClass = gestureClass;
		
		if(classified){
			token.setGestureClass(gestureClass);
	//		if(!gestureClass.getDefinition().isParameter()) 
	//			token.setOperation(gestureClass.getDefinition().getOperation());
	//		else token.setParameter(gestureClass.getDefinition().getName());
				
			if(token.getAlternativeScopes().size() > 1){
				token.setActiveScope(token.getAlternativeScopes().getScope(gestureClass.getDefinition().getType()));
			
				setScopeTo(token.getScope());
				for(GesturePath gpath:token.getOtherPaths(this))
					gpath.setScopeTo(token.getScope());
					
			}
		}
		else {
			token.setGestureClass(null);
		}
	
/*		
		if(toAddButton) {
			if(samplesButton != null) samplesButton.update();
			else samplesButton = new AddToSamplesButton(this);
		}
		else if(newClass && samplesButton != null ||
				samplesButton!=null && gestureClass == null){
			removeChild(samplesButton);
			samplesButton = null;
		}
*/		
	}
	
	public GeneralizedGestureClass getGestureClass(){
		return gestureClass;
	}

	public Collection<GeneralizedGestureClass> getGestureClasses(){
		Vector<GeneralizedGestureClass> classes = new Vector<GeneralizedGestureClass>();
		
		if(gestureClass != null){
			classes.add(gestureClass);
			return classes;
		}
		
		Collection<Token> tokens = getToken().getSubTokens();
		if(getToken() instanceof ConnectedComplexGesture || getToken() instanceof AssociatedComplexGesture){ 
			for(Token token:tokens) {
				Collection<GeneralizedGestureClass> newclasses = token.getGesturePath(gcanvas).getGestureClasses();
				if(newclasses!=null) classes.addAll(newclasses);
			}
			
			return classes;
		}
		
		return null;
	}
	
	
	public boolean isDisabled(){
		return disabled;
	}
	
	
	public void setDisabled() {
		disabled = true;
		
		if(token.getParent() != null) return;
		
		fixColor(PAINT_DISABLED, STROKE_DISABLED, null);	
//		if(samplesButton != null) samplesButton.setVisible(false);
	}
	
	List<GeneralizedGestureClass> classList  = new Vector<GeneralizedGestureClass> ();
	public void addPossibleClasses(List<GeneralizedGestureClass> completeResult) {		

		this.classList = completeResult;			
		
		if(classList == null) classList = new Vector<GeneralizedGestureClass> ();
	}
	
	public List<GeneralizedGestureClass> getAlternativeResults(){	
	
		if(gestureClass!=null && classList != null) {
			
			Vector<GeneralizedGestureClass> tmpList = new Vector<GeneralizedGestureClass>();
			for(GeneralizedGestureClass gclass:classList){		
				if(gclass!=gestureClass) tmpList.add(gclass);
			}
			
			return tmpList;
		}
		else return classList;
	}


}
