package inria.paperoles.ui;

import inria.paperoles.customization.ClassifyAllCommand;
import inria.paperoles.customization.GestureManipulationCommand;
import inria.paperoles.customization.GestureRecognitionCommand;
import inria.paperoles.customization.ParameterRecognitionCommand;
import inria.paperoles.customization.QueryCommand;
import inria.paperoles.docmodel.Symbol;
import inria.paperoles.gestures.SingleInkStroke;

import inria.paperoles.gestures.StrokeAnalyzer;
import inria.paperoles.gestures.recognition.RecognitionManager;

import inria.paperoles.grammar.ComplexGesture;
import inria.paperoles.grammar.GroupingGesture;
import inria.paperoles.grammar.Horizontal;
import inria.paperoles.grammar.PrintedScope;
import inria.paperoles.grammar.SymbolTable;
import inria.paperoles.grammar.SyntaxAnalyzer;
import inria.paperoles.grammar.Token;
import inria.paperoles.grammar.Unknown;
import inria.paperoles.grammar.WrittenScope;
import inria.paperoles.scoremodel.GestureNote;
import inria.paperoles.scoremodel.Note;
import inria.paperoles.scoremodel.ScoreSkeleton;
import inria.paperoles.scoremodel.WavePoint;
import inria.paperoles.uievents.GestureEventHandler;

import java.awt.*;
import java.util.Collection;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Vector;

import papertoolkit.pen.PenSample;
import papertoolkit.pen.ink.InkStroke;
import edu.umd.cs.piccolo.PCamera;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PPath;
import edu.umd.cs.piccolo.util.PBounds;

public class GesturesCanvas extends PPath{
	
	GestureEventHandler eventHandler;
	Point viewPosition = new Point(0, 0);
	
	//ScopeGestureSet lastGestureSet;
	StrokeAnalyzer strokeAnalyzer;
	SyntaxAnalyzer syntaxAnalyzer;
	
	ScoreSkeleton pageSkeleton;
	
	int reconDistanceThreshold = GlobalParameters.DEFAULT_RECOGNIZER_DISTANCE;
	

	public GesturesCanvas(ScoreSkeleton skeleton){	
		eventHandler = new GestureEventHandler(this);
		pageSkeleton = skeleton;
		skeleton.setCanvas(this);	
	}
	
	public ScoreSkeleton getPageSkeleton(){
		return pageSkeleton;
	}
	
	public void setAnalysisHandlers(StrokeAnalyzer strokeAnalyzer, SyntaxAnalyzer syntaxAnalyzer){
		this.strokeAnalyzer = strokeAnalyzer;	
		this.syntaxAnalyzer = syntaxAnalyzer;
	}
	
	public ScoreViewer getViewer(){
		return ((ScoreViewer)((PCamera)getRoot().getChild(0)).getComponent());
	}
	
	public void addNewStroke(SingleInkStroke stroke, boolean realPenData){//System.err.println(stroke);
		
		SingleInkStroke transStroke = realPenData?stroke:transformInkStroke(stroke, getViewScale(), viewPosition);
			
		/*
		if(pageSkeleton.inNotationBox(transStroke)){
			strokeAnalyzer.setToNotationMode();
				
			return;
		}
		else if(pageSkeleton.inAnnotationBox(transStroke)){
			strokeAnalyzer.setToAnnotationMode();
			
			return;
		}*/
		
		strokeAnalyzer.performAnalysis(transStroke, pageSkeleton);
		
		/*
		if(strokeAnalyzer.inNotationMode()) {
			Symbol gnote = strokeAnalyzer.processNotationGesture(stroke, pageSkeleton);
			
			// OK, what is this here?
			notationCanvas.addChild((PNode)gnote.clone());	
			
			return;
		}*/
		
		
		Token token = syntaxAnalyzer.analyzeNewStroke(stroke, this);
		
		GesturePath gpath = new GesturePath(this, token); 
		addChild(gpath);
		token.addGesture(gpath);
	}
	
	
	private void rebuild(){
	//	syntaxAnalyzer.recognizeSyntax();
		removeLastChildren();
	
		SymbolTable symbolTable = syntaxAnalyzer.getSymbolTable();
	
		Vector<GesturePath> paths = new Vector<GesturePath>();
		
		for(Token token:symbolTable.getTokens()){
		//	if(token.getCanvas() == this) 
	
			if(!token.hasStrokeIn(this) && !token.hasSkopeIn(this)) continue;
	
			GesturePath gpath = new GesturePath(this, token);
			addChild(gpath);
			token.addGesture(gpath);
			
			gpath.addInputEventListener(eventHandler);
			paths.add(gpath);
		}
		
		for(GesturePath gpath:paths) recognizeGesture(gpath);
		
	}
	
	public void cleanSpace(){
		removeAllChildren();
		lastChildrenIndex = 0;
	}
	
	public void cleanRecognition(){
		for(int i=0; i<getChildrenCount(); ++i){ 
			recognizeGesture(((GesturePath) getChild(i)));
		}
	}
	
	
	private void recognizeGesture(GesturePath gpath){
			GestureManipulationCommand command = null;
			
			if(gpath.getToken() instanceof GroupingGesture) 
				command = new ParameterRecognitionCommand(gpath);
			else if(gpath.getToken() instanceof ComplexGesture){
				Vector<Token> subtokens = ((ComplexGesture)gpath.getToken()).getSubTokens();
				for(int i=1; i<subtokens.size(); ++ i)		
					recognizeGesture(subtokens.elementAt(i).getGesturePath(gpath.getCanvas()));	
				
				return;
			}
			else command = new GestureRecognitionCommand(gpath);

			command.activate(null);
	}

	int lastChildrenIndex = 0;
	private void removeLastChildren(){ 
		Vector<GesturePath> childrenToRemove = new Vector<GesturePath>();
		for(int i=lastChildrenIndex; i<getChildrenCount(); ++i){ 
			childrenToRemove.add((GesturePath) getChild(i)); 
		}
		
		removeChildren(childrenToRemove);
	}
	
	public void refresh(){
		rebuild();
		lastChildrenIndex = getChildrenCount();
	}
	
	public void setInWriting(boolean inWriting){
		if(inWriting){
			lastChildrenIndex = getChildrenCount();
	//		syntaxAnalyzer.renew();
		}
		else rebuild();
	}
	
	
	private SingleInkStroke transformInkStroke(SingleInkStroke stroke, double scale, Point offset){
		java.util.Collection<PenSample> samples = stroke.getSamples();

		for(PenSample sample: samples){
			sample.setX((sample.getX()+offset.getX())/scale);
			sample.setY((sample.getY()+offset.getY())/scale);
		}
		
		return stroke;

	}
	
	public double getViewScale(){	
		return ((PLayer)getParent()).getCamera(0).getViewScale();
	}
	
	public void setViewPosition(Point viewPosition){
		this.viewPosition = viewPosition;
	}
	
	
	float[] convertToFloat(double[] darray){
		float[] farray = new float[darray.length];
		for(int i=0; i<farray.length; ++i)
			farray[i] = (float)darray[i];
		
		return farray;		
	}

	ScopeMenu smenu;
	public void addScopeMenu(GesturePath affectedPath, ScopeMenu scopeMenu) {
	/*	for(Object node:getChildrenReference()){
			GesturePath path = (GesturePath)node;
			if(path != affectedPath) path.setDisabled();
		}
		*/
		addChild(scopeMenu);
		smenu = scopeMenu;
	}

	public SyntaxAnalyzer getSyntaxAnalyzer(){
		return syntaxAnalyzer;
	}
	
	public void removeScopeMenu(){
		smenu.dismiss();
		removeChild(smenu);
	}

	public void setDistanceThreshold(int value) {
		reconDistanceThreshold = value;
		
		Iterator childIterator = getChildrenIterator();
		while(childIterator.hasNext())
			((GesturePath)childIterator.next()).applyNewThresDistance();	
	}

	
	/*
	HandwrittenNotationCanvas notationCanvas;
	public void setNotationLayer(HandwrittenNotationCanvas notationCanvas) {
		this.notationCanvas = notationCanvas; 
		
	}*/

	
	public Collection<Token> getMatchedTokens() {		
		TreeMap<Integer, Token> tokens = new TreeMap<Integer, Token>();
		
		Iterator childIterator = getChildrenIterator();
		while(childIterator.hasNext()){
			Token token = ((GesturePath)childIterator.next()).getToken();	
			if(token.getOperation() == null) continue;
			
			tokens.put(token.getTimePosition(), token);
		}
		
		return tokens.values();
	}
	
	
	public void addToOperations(int timePosition) {
		
	}

	public void removeFromOperations(Token token) {
		// TODO Auto-generated method stub
		
	}
	
}
