package inria.paperoles.ui;

import inria.paperoles.customization.GestureFilteringCommand;
import inria.paperoles.gestures.definition.GestureDefinition;
import inria.paperoles.gestures.recognition.GeneralizedGestureClass;
import inria.paperoles.operations.Operation;
import inria.paperoles.operations.OperationArgument;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JList;
import javax.swing.JLabel;
import javax.swing.ListCellRenderer;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.ximtec.igesture.core.GestureClass;

import sun.tools.jar.JarImageSource;

public class OperationsList extends JList implements ListSelectionListener{

	GestureFilteringCommand filteringCommand;
	//Hashtable<String, Operation> operations;
	
	SortedMap<String, Operation> operations;
	
	public OperationsList(final GesturesContainer gcontainer){
		super(new DefaultListModel());
		
		setCellRenderer(new OperationCellRenderer());
		setVisibleRowCount(-1);
		setLayoutOrientation(JList.VERTICAL);
		
		filteringCommand = new GestureFilteringCommand(gcontainer);
		addListSelectionListener(this);
		addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e) {
				if(e.getClickCount() < 2) return;
				
				editSelectedOperation(gcontainer.getBrowserFrame(), 
						e.getXOnScreen(), e.getYOnScreen());
			}
		});
		
		operations = new TreeMap<String, Operation>();
		
		/*
		Operation operation = new Operation("Remove");
		operation.addArgument(new OperationArgument(operation, Operation.SCORE_SCOPE));
		addOperation(operation);
		addOperation(new Operation("Copy-Paste"));
		*/
	}
	
	public int getOperationIndex(String operationName){
		if(operationName == null) return -1;
		
		Collection<Operation> ops = operations.values();
		
		Operation operat = operations.get(operationName);	
		
		// in case the name of the operation has changed...
		if(!operat.getName().equals(operationName)){
			operations.remove(operationName);
			operations.put(operat.getName(), operat);
		}
		
		int i = 0;
		for(Operation op:ops){
			if(op == operat) return i;
			++i;
		}
				
		return -1;
	}
	
	Operation saveOperation = null;
	private void editSelectedOperation(GestureBrowser browser, int x, int y){
		ListItem selection = (ListItem)getSelectedValue();
		
		saveOperation = new Operation(selection.getOperation());
		(new OperationDefinitionDialog(browser, x, y, selection.getOperation())).setVisible(true);
		saveOperation = null;
	}
	
	public void cancelEdit(){
		((ListItem)getSelectedValue()).operation = saveOperation;
	}
	
	public void addOperation(Operation operation){ 
		ListItem listItem = new ListItem(operation);
		operations.put(operation.getName().toLowerCase(), operation);
		
		int i = 0;
		for(; i< ((DefaultListModel)getModel()).getSize(); ++i){
			ListItem item = (ListItem)((DefaultListModel)getModel()).getElementAt(i);
			if(listItem.getLabel().compareTo(item.getLabel()) < 0) break;
		}
		
		((DefaultListModel)getModel()).insertElementAt(listItem, i);
	}
	

	public Vector<Operation> getOperationsVector(){
		Vector<Operation> operations = new Vector<Operation>();
			
		for(int i =0; i< ((DefaultListModel)getModel()).getSize(); ++i){
			ListItem item = (ListItem)((DefaultListModel)getModel()).getElementAt(i);
			operations.add(item.getOperation());
		}
		
		return operations;
	}
	
	final static Color selectColor = GlobalParameters.LIST_SELECTION_COLOR;
		
	private class OperationCellRenderer extends JLabel implements ListCellRenderer{
		public OperationCellRenderer(){
			super();
			setFont(new Font("Arial", Font.PLAIN, 10));
			setForeground(Color.black);
			setOpaque(true);
	       	setVerticalTextPosition(JLabel.BOTTOM);
        	setHorizontalTextPosition(JLabel.CENTER);
    		setBorder(BorderFactory.createEmptyBorder(3,3,3,3));
		}

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
        {
        	
        	ListItem item = (ListItem)value;
            setText(item.getLabel());

            setBackground(isSelected ?  selectColor: Color.white);

            return this;
        }
	}
	
    private class ListItem
    {
    	
    	Operation operation;
        
        ListItem(Operation operation)
        {
            this.operation = operation;
        }
  
        public String getLabel(){
        	return operation.definition();
        }
        
        public Operation getOperation(){
        	return operation;
        }
        
    }

    public SortedMap<String, Operation> getOperationsMap(){
    	return operations;
    }
    
    public Collection<Operation> getOperations(){
    	return operations.values();
    }
    
	public void valueChanged(ListSelectionEvent e) {

	}

	public boolean contains(String s) {
		Operation op = operations.get(s.toLowerCase());
		if(op!=null && (getSelectedValue() == null || op != ((ListItem)getSelectedValue()).operation)) return true; 
		else return false;
	}

	public void loadOperations(Vector<Operation> operations) {
		
		this.operations.clear();
		((DefaultListModel)getModel()).clear();
				
		for(Operation operation:operations){
			addOperation(operation);
		}
		
		//validate();
	}
}
