package inria.paperoles.ui;

import inria.paperoles.grammar.SyntaxAnalyzer;
import inria.paperoles.grammar.Token;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Iterator;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class RubineController extends JSlider implements ChangeListener{

	final static int MIN = 50;
	final static int MAX = GlobalParameters.MAX_RECOGNIZER_DISTANCE;
	GesturesContainer gcontainer;
	SyntaxAnalyzer analyzer;
	
	public RubineController(GesturesContainer gcontainer, int defaultValue){
		super(JSlider.VERTICAL, MIN, MAX, defaultValue);
		
//		this.analyzer = analyzer;
		this.gcontainer = gcontainer;
		setMinorTickSpacing(300);
//		setMinorTickSpacing(20);
		
		setPaintTicks(true);
		setAlignmentX(JSlider.CENTER_ALIGNMENT);
		setMaximumSize(new Dimension(40, 120));
		setSnapToTicks(true);
		setToolTipText("Set the sensitivity of the gesture recognizer");
		setBorder(BorderFactory.createLineBorder(Color.gray));
		
		value = getValue();
		
		addChangeListener(this);
		addMouseListener(new MouseAdapter(){
			public void mouseEntered(MouseEvent e){
//setToolTipText(": " + getValue());
				entered();
			}
			
			public void mouseExited(MouseEvent e){
				exited();
			}
		});
	}
	
	public void setSyntaxAnalyzer(SyntaxAnalyzer sanalyzer){
		analyzer = sanalyzer;
	}

	int enteredValue = -1;
	void entered(){
		enteredValue = getValue();
	}
	
	void exited(){
		Set<Token> tokens;
		
		if(enteredValue == getValue() || analyzer == null) return;
		
		tokens = gcontainer.updateClasses();
/*		Iterator<Token> tokenIterator = tokens.iterator();
		
		while(tokenIterator.hasNext())
			analyzer.matchOperations(tokenIterator.next());
*/
	}
	
	int value;
	public void stateChanged(ChangeEvent e) {
		if(getValue() == value) return;
	
		value = getValue();
		gcontainer.setDistanceThreshold(value);
	}
	
	
}
