package inria.paperoles.ui;

import inria.paperoles.grammar.AlternativeScopes;
import inria.paperoles.grammar.PrintedScope;
import inria.paperoles.grammar.Scope;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.SortedSet;

import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PPath;

public class ScopeMenu extends PPath{

	final static Color NORMAL_STROKE = Color.red;//new Color(100,20,20);
	final static Color FOCUS_STROKE = Color.blue;
	
	final static Color NORMAL_PAINT = new Color(255, 100, 100, 5);
	final static Color FOCUS_PAINT = new Color(100, 110, 220, 100);
	
	PPath mask;
	ScopeOption activeOption = null;

	//ScopeOption defaultOption = null;
	GesturePath gpath;
	
	public ScopeMenu(GesturePath gpath){
		
		AlternativeScopes altscopes = gpath.getToken().getAlternativeScopes();
		
		for(Scope scope:altscopes.getScopes()){ 
			ScopeOption option = new ScopeOption(scope);
			addChild(option);

	/*		if(scope.equals(gpath.getToken().getScope())){ 
				defaultOption = option;
				activeOption = option;
				
				
				showMask(option.shape);
				option.setFocus(true);
			}*/
		}
		
		gpath.getParent().setPickable(true);
		this.gpath = gpath;
		gpath.scope.setVisible(false);
	}
	
	public void showMask(Shape shape){
		if(mask == null){
			mask = new PPath();
			addChild(mask);
			mask.setStrokePaint(FOCUS_STROKE);
			mask.setStroke(new BasicStroke(2.0f));/*,  // Width
                    BasicStroke.CAP_SQUARE,    // End cap
                    BasicStroke.JOIN_MITER    // Join style*/
                   // 10.0f,                     // Miter limit
                   // new float[] {10.0f,10.0f}, // Dash pattern
                    /*0.0f));*/  
			mask.setPickable(false);
		}
		
		mask.setPathTo(shape);
		if(!mask.getVisible()) mask.setVisible(true);	
	}
	
	public void hideMask(){
		mask.setVisible(false);
	}
	
	class ScopeOption extends PPath{
		Scope scope;
		Shape shape;
		
		public ScopeOption(Scope scope){
			super(scope.getShape());
			this.shape = scope.getShape();		
			
			setFocus(false);
			setStrokePaint(NORMAL_STROKE);
			
			if(scope instanceof PrintedScope){
				setStroke(new BasicStroke(1.0f,     // Width
                    BasicStroke.CAP_SQUARE,    // End cap
                    BasicStroke.JOIN_MITER,    // Join style
                    10.0f,                     // Miter limit
                    new float[] {5.0f,5.0f}, // Dash pattern
                    0.0f));                     // Dash phase);
			}

			
			this.scope = scope;
		}
		
		public boolean contains(Point2D p){
			if(shape./*getBounds().*/contains(p)) return true;
			else return false;
		}
		
		public double getDistance(Point2D p){
			Rectangle bounds = shape.getBounds();
			if(bounds.contains(p)) return 0;
			
			double mindist = 400000, dist;
			Line2D line;
			
			if(p.getY() <= bounds.getMinY()){
				line = new Line2D.Double(bounds.getMinX(), bounds.getMinY(),
					bounds.getMaxX(), bounds.getMinY());
				mindist = line.ptSegDistSq(p);
			}
			
			if(p.getX() >= bounds.getMaxX()){
				line = new Line2D.Double(bounds.getMaxX(), bounds.getMinY(),
					bounds.getMaxX(), bounds.getMaxY());
				if((dist = line.ptSegDistSq(p)) < mindist) mindist = dist;
			}
			
			if(p.getY() >= bounds.getMaxY()){
				line = new Line2D.Double(bounds.getMaxX(), bounds.getMaxY(),
					bounds.getMinX(), bounds.getMaxY());
				if((dist = line.ptSegDistSq(p)) < mindist) mindist = dist;		
			}
			
			if(p.getX()< bounds.getMinX()){
				line = new Line2D.Double(bounds.getMinX(), bounds.getMaxY(),
					bounds.getMinX(), bounds.getMinY());
				if((dist = line.ptSegDistSq(p)) < mindist) mindist = dist;			
			}
			
			return mindist;
		}
		
		public void setFocus(boolean focus){
			if(focus){
				setPaint(FOCUS_PAINT);
			}
			else {
				setPaint(NORMAL_PAINT);
			}
		}
	}

	public void handleEvent(PInputEvent event) {
		
		ScopeOption closestOption = null;
		
		for(int i = getChildrenCount() - 1; i>=0; --i){
			PNode option = getChild(i);
			if(!(option instanceof ScopeOption)) continue;
				
			if(((ScopeOption)option).contains(event.getPosition())){
				closestOption = (ScopeOption)option;
				break;
			}
				
		}
		
		if(closestOption == activeOption) return;
		if(activeOption!=null) activeOption.setFocus(false);
		
		activeOption = closestOption;
		if(closestOption !=null){
			closestOption.setFocus(true);
			showMask(activeOption.shape);
		}
		else hideMask();
	
	}

	public void dismiss() {
		if(activeOption!=null){
			gpath.setScopeTo(activeOption.scope);
			gpath.getToken().setActiveScope(activeOption.scope);
		}
		gpath.setNormal();
		gpath.scope.setVisible(true);
	}
}
